@@ -21,7 +21,7 @@
    
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
    */
 
 #ifndef _GNUstep_H_GSDrawFunctions
@@ -35,6 +35,9 @@
 
 @interface GSDrawFunctions : NSObject
 
++ (id) theme;
++ (void) setTheme: (id)theme;
+
 + (NSRect) drawButton: (NSRect)border : (NSRect)clip;
 + (NSRect) drawDarkButton: (NSRect)border : (NSRect)clip;
 + (NSRect) drawDarkBezel: (NSRect)border : (NSRect)clip;
@@ -48,6 +51,19 @@
            inRect: (NSRect)border 
            withClip: (NSRect)clip;
 
+- (NSRect) drawButton: (NSRect)border : (NSRect)clip;
+- (NSRect) drawDarkButton: (NSRect)border : (NSRect)clip;
+- (NSRect) drawDarkBezel: (NSRect)border : (NSRect)clip;
+- (NSRect) drawLightBezel: (NSRect)border : (NSRect)clip;
+- (NSRect) drawGrayBezel: (NSRect)border : (NSRect)clip;
+- (NSRect) drawWhiteBezel: (NSRect)border : (NSRect)clip;
+- (NSRect) drawGroove: (NSRect)border : (NSRect)clip;
+- (NSRect) drawFramePhoto: (NSRect)border : (NSRect)clip;
+
+- (NSRect) drawGradientBorder: (NSGradientType)gradientType 
+           inRect: (NSRect)border 
+           withClip: (NSRect)clip;
+
 @end
 
 #endif /* _GNUstep_H_GSDrawFunctions */
