@@ -21,7 +21,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */ 
 
 #ifndef _GNUstep_H_GSServicesManager
@@ -61,6 +61,7 @@
   NSMutableSet		*_allDisabled;
   NSMutableDictionary	*_allServices;
   NSTimer		*_timer;
+  NSString		*_port;
 }
 + (GSServicesManager*) newWithApplication: (NSApplication*)app;
 + (GSServicesManager*) manager;
@@ -78,6 +79,7 @@
 - (NSString*) item2title: (id<NSMenuItem>)item;
 - (void) loadServices;
 - (NSDictionary*) menuServices;
+- (NSString*) port;
 - (void) rebuildServices;
 - (void) rebuildServicesMenu;
 - (void) registerAsServiceProvider;
