@@ -23,7 +23,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */ 
 
 #ifndef _GNUstep_H_NSDataLinkManager
@@ -44,12 +44,16 @@
   // Attributes
   id                  delegate;
   NSString            *filename;
-  BOOL                delegateVerifiesLinks;
-  BOOL                interactsWithUser;
-  BOOL                isEdited;
-  BOOL                areLinkOutlinesVisible;
   NSMutableArray      *sourceLinks;
   NSMutableArray      *destinationLinks;
+
+  struct __dlmFlags {
+    unsigned areLinkOutlinesVisible:1;
+    unsigned delegateVerifiesLinks:1;
+    unsigned interactsWithUser:1;
+    unsigned isEdited:1;
+  } _flags;
+
 }
 
 //
