@@ -37,7 +37,8 @@
 @class NSAttributedString;
 
 @interface NSBundle (NSHelpManager)
-- (NSAttributedString*) contextHelpForKey: (NSString*) key;
+- (NSString *)pathForHelpResource:(NSString *)fileName;
+- (NSAttributedString *)contextHelpForKey:(NSString *)key;
 @end
 
 @interface NSApplication (NSHelpManager)
@@ -67,6 +68,8 @@
 
 - (void) removeContextHelpForObject: (id)object;
 
+- (void)setContextHelp:(NSAttributedString *)help forObject:(id)object;
+
 - (void) setContextHelp: (NSAttributedString*) help withObject: (id) object;
 
 - (BOOL) showContextHelpForObject: (id)object locationHint: (NSPoint) point;
