@@ -23,7 +23,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */ 
 
 #ifndef _GNUstep_H_NSResponder
@@ -169,6 +169,8 @@
  */
 - (NSInterfaceStyle) interfaceStyle;
 - (void) setInterfaceStyle: (NSInterfaceStyle)aStyle;
+
+- (BOOL) shouldBeTreatedAsInkEvent: (NSEvent *)theEvent;
 #endif
 @end
 
@@ -232,6 +234,13 @@
 - (void) transposeWords: (id)sender;
 - (void) uppercaseWord: (id)sender;
 - (void) yank: (id)sender;
+
+// New in MacOSX 10.3
+- (void) cancelOperation: (id)sender;
+- (void) deleteBackwardByDecomposingPreviousCharacter: (id)sender;
+- (void) moveLeftAndModifySelection: (id)sender;
+- (void) moveRightAndModifySelection: (id)sender;
+
 @end
 #endif
 
