@@ -80,7 +80,7 @@ GSServerForWindow(NSWindow *window)
       /* Backend window hasn't been initialized yet, assume current server. */
       return GSCurrentServer();
     }
-  return NSMapGet(windowmaps, (void *)num);
+  return NSMapGet(windowmaps, (void *)(intptr_t)num);
 }
 
 /** Returns the current GSDisplayServer */
@@ -511,7 +511,7 @@ GSCurrentServer(void)
     for a window that has already been created */
 - (void) _setWindowOwnedByServer: (int)win
 {
-  NSMapInsert (windowmaps, (void*)win,  self);
+  NSMapInsert (windowmaps, (void*)(intptr_t)win,  self);
 }
 
 /** Creates a window whose location and size is described by frame and
@@ -547,6 +547,15 @@ GSCurrentServer(void)
   [self subclassResponsibility: _cmd];
 }
 
+/** Create all the backend structures for a reference to a native window and 
+    return the extend, backing type, style and screen for that window. */ 
+- (int) nativeWindow: (void *)winref : (NSRect*)frame : (NSBackingStoreType*)type 
+		    : (unsigned int*)style : (int*)screen
+{
+  [self subclassResponsibility: _cmd];
+  return 0;
+}
+
 /** Sets the style of the window. See [NSWindow-styleMask] for a
     description of the available styles */
 - (void) stylewindow: (unsigned int) style : (int) win
