@@ -69,6 +69,7 @@
 	position: (NSPoint)eventLocation
        timestamp: (NSTimeInterval)time
 	toWindow: (NSWindow*)dWindow;
+- (void) _handleDrag: (NSEvent*)theEvent slidePoint: (NSPoint)slidePoint;
 - (void) _handleEventDuringDragging: (NSEvent *)theEvent;
 - (void) _updateAndMoveImageToCorrectPosition;
 - (void) _moveDraggedImageToNewPosition;
@@ -110,7 +111,7 @@
 
 static	GSDragView *sharedDragView = nil;
 
-+ (GSDragView*) sharedDragView
++ (id) sharedDragView
 {
   if (sharedDragView == nil)
     {
@@ -306,6 +307,32 @@ static	GSDragView *sharedDragView = nil;
     }
 }
 
+- (void) sendExternalEvent: (GSAppKitSubtype)subtype
+		    action: (NSDragOperation)action
+		  position: (NSPoint)eventLocation
+		 timestamp: (NSTimeInterval)time
+		  toWindow: (int)dWindowNumber
+{
+}
+
+/*
+  Return the window that lies below the cursor and accepts drag and drop.
+  In mouseWindowRef the OS reference for this window is returned, this is even 
+  set, if there is a native window, but no GNUstep window at this location.
+ */
+- (NSWindow*) windowAcceptingDnDunder: (NSPoint)mouseLocation
+			    windowRef: (int*)mouseWindowRef
+{
+  int win;
+
+  *mouseWindowRef = 0;
+  win = [GSServerForWindow(_window) findWindowAt: mouseLocation
+				       windowRef: mouseWindowRef
+				       excluding: [_window windowNumber]];
+
+  return GSWindowWithNumber(win);
+}
+
 @end
 
 @implementation GSDragView (Private)
@@ -516,7 +543,7 @@ static	GSDragView *sharedDragView = nil;
   NSEvent *e;
   NSGraphicsContext *context = GSCurrentContext();
   // FIXME: Should store this once
-  int dragWindowRef = (int)[GSServerForWindow(_window) windowDevice: [_window windowNumber]];
+  int dragWindowRef = (int)(intptr_t)[GSServerForWindow(_window) windowDevice: [_window windowNumber]];
 
   eventLocation = [dWindow convertScreenToBase: eventLocation];
   e = [NSEvent otherEventWithType: NSAppKitDefined
@@ -531,14 +558,6 @@ static	GSDragView *sharedDragView = nil;
   [dWindow sendEvent: e];
 }
 
-- (void) sendExternalEvent: (GSAppKitSubtype)subtype
-		    action: (NSDragOperation)action
-		  position: (NSPoint)eventLocation
-		 timestamp: (NSTimeInterval)time
-		  toWindow: (int)dWindowNumber
-{
-}
-
 /*
   The dragging support works by hijacking the NSApp event loop.
 
@@ -980,22 +999,4 @@ static	GSDragView *sharedDragView = nil;
     }
 }
 
-/*
-  Return the window that lies below the cursor and accepts drag and drop.
-  In mouseWindowRef the OS reference for this window is returned, this is even 
-  set, if there is a native window, but no GNUstep window at this location.
- */
-- (NSWindow*) windowAcceptingDnDunder: (NSPoint)mouseLocation
-			    windowRef: (int*)mouseWindowRef
-{
-  int win;
-
-  *mouseWindowRef = 0;
-  win = [GSServerForWindow(_window) findWindowAt: mouseLocation
-				       windowRef: mouseWindowRef
-				       excluding: [_window windowNumber]];
-
-  return GSWindowWithNumber(win);
-}
-
 @end
