@@ -21,7 +21,7 @@
    
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
    */
 
 #include "GNUstepGUI/GSDrawFunctions.h"
@@ -50,8 +50,83 @@
 */ 
 @implementation GSDrawFunctions
 
-/** Draw a button border */
+static id theTheme = nil;
+
+
++ (id) theme 
+{
+  if (theTheme == nil)
+    {
+      theTheme = [self new];
+    }
+  return theTheme;
+}
+
++ (void) setTheme: (id) aTheme
+{
+  ASSIGN (theTheme, aTheme);
+}
+
+/** Tell current theme to draw a button border */
 + (NSRect) drawButton: (NSRect)border : (NSRect)clip
+{
+  return [[self theme] drawButton: border : clip];
+}
+
+/** Tell current theme to draw a "dark" button border (used in tableviews) */
++ (NSRect) drawDarkButton: (NSRect)border : (NSRect)clip
+{
+  return [[self theme] drawDarkButton: border : clip];
+}
+
+/** Tell current theme to draw a dark bezel border */
++ (NSRect) drawDarkBezel: (NSRect)border : (NSRect)clip
+{
+  return [[self theme] drawDarkBezel: border : clip];
+}
+
+/** Tell current theme to draw a light bezel border */
++ (NSRect) drawLightBezel: (NSRect)border : (NSRect)clip
+{
+  return [[self theme] drawLightBezel: border : clip];
+}
+
+/** Tell current theme to draw a white bezel border */
++ (NSRect) drawWhiteBezel: (NSRect)border : (NSRect)clip
+{
+  return [[self theme] drawWhiteBezel: border : clip];
+}
+
+/** Tell current theme to draw a grey bezel border */
++ (NSRect) drawGrayBezel: (NSRect)border : (NSRect)clip
+{
+  return [[self theme] drawGrayBezel: border : clip];
+}
+
+/** Tell current theme to draw a groove border */
++ (NSRect) drawGroove: (NSRect)border : (NSRect)clip
+{
+  return [[self theme] drawGroove: border : clip];
+}
+
+/** Tell current theme to draw a frame photo border.  Used in NSImageView.   */
++ (NSRect) drawFramePhoto: (NSRect)border : (NSRect)clip
+{
+  return [[self theme] drawFramePhoto: border : clip];
+}
+
+/** Tell current theme to draw a gradient border. */
++ (NSRect) drawGradientBorder: (NSGradientType)gradientType 
+		       inRect: (NSRect)border 
+		     withClip: (NSRect)clip
+{
+  return [[self theme] drawGradientBorder: gradientType
+				   inRect: border
+				 withClip: clip];
+}
+
+
+- (NSRect) drawButton: (NSRect)border : (NSRect)clip
 {
   NSRectEdge up_sides[] = {NSMaxXEdge, NSMinYEdge, 
 			   NSMinXEdge, NSMaxYEdge, 
@@ -77,7 +152,7 @@
 }
 
 /** Draw a "dark" button border (used in tableviews) */
-+ (NSRect) drawDarkButton: (NSRect)border : (NSRect)clip
+- (NSRect) drawDarkButton: (NSRect)border : (NSRect)clip
 {
   NSRectEdge up_sides[] = {NSMaxXEdge, NSMinYEdge, 
 			   NSMinXEdge, NSMaxYEdge}; 
@@ -99,7 +174,7 @@
 }
 
 /** Draw a dark bezel border */
-+ (NSRect) drawDarkBezel: (NSRect)border : (NSRect)clip
+- (NSRect) drawDarkBezel: (NSRect)border : (NSRect)clip
 {
   NSRectEdge up_sides[] = {NSMaxXEdge, NSMinYEdge, NSMinXEdge, NSMaxYEdge,
 			   NSMinXEdge, NSMaxYEdge, NSMaxXEdge, NSMinYEdge};
@@ -134,7 +209,7 @@
 }
 
 /** Draw a light bezel border */
-+ (NSRect) drawLightBezel: (NSRect)border : (NSRect)clip
+- (NSRect) drawLightBezel: (NSRect)border : (NSRect)clip
 {
   NSRectEdge up_sides[] = {NSMaxXEdge, NSMinYEdge, NSMinXEdge, NSMaxYEdge, 
   			   NSMaxXEdge, NSMinYEdge, NSMinXEdge, NSMaxYEdge};
@@ -158,7 +233,7 @@
 }
 
 /** Draw a white bezel border */
-+ (NSRect) drawWhiteBezel: (NSRect)border : (NSRect)clip
+- (NSRect) drawWhiteBezel: (NSRect)border : (NSRect)clip
 {
   NSRectEdge up_sides[] = {NSMaxYEdge, NSMaxXEdge, NSMinYEdge, NSMinXEdge,
   			   NSMaxYEdge, NSMaxXEdge, NSMinYEdge, NSMinXEdge};
@@ -182,7 +257,7 @@
 }
 
 /** Draw a grey bezel border */
-+ (NSRect) drawGrayBezel: (NSRect)border : (NSRect)clip
+- (NSRect) drawGrayBezel: (NSRect)border : (NSRect)clip
 {
   NSRectEdge up_sides[] = {NSMaxXEdge, NSMinYEdge, NSMinXEdge, NSMaxYEdge,
 			   NSMaxXEdge, NSMinYEdge, NSMinXEdge, NSMaxYEdge};
@@ -215,7 +290,7 @@
 }
 
 /** Draw a groove border */
-+ (NSRect) drawGroove: (NSRect)border : (NSRect)clip
+- (NSRect) drawGroove: (NSRect)border : (NSRect)clip
 {
   // go clockwise from the top twice -- makes the groove come out right
   NSRectEdge up_sides[] = {NSMaxYEdge, NSMaxXEdge, NSMinYEdge, NSMinXEdge,
@@ -239,7 +314,7 @@
 }
 
 /** Draw a frame photo border.  Used in NSImageView.   */
-+ (NSRect) drawFramePhoto: (NSRect)border : (NSRect)clip
+- (NSRect) drawFramePhoto: (NSRect)border : (NSRect)clip
 {
   NSRectEdge up_sides[] = {NSMaxXEdge, NSMinYEdge, 
 			   NSMinXEdge, NSMaxYEdge, 
@@ -264,7 +339,7 @@
 }
 
 /** Draw a gradient border. */
-+ (NSRect) drawGradientBorder: (NSGradientType)gradientType 
+- (NSRect) drawGradientBorder: (NSGradientType)gradientType 
 		       inRect: (NSRect)border 
 		     withClip: (NSRect)clip
 {
