@@ -161,8 +161,14 @@ NSRegisterServicesProvider(id provider, NSString *name)
       DESTROY(listenerConnection);
     }
 
-  listenerConnection = [NSConnection newRegisteringAtName: name
-    withRootObject: [GSListener listener]];
+  listenerConnection = [[NSConnection alloc]
+    initWithReceivePort: [NSPort port] sendPort: nil];
+  [listenerConnection setRootObject: [GSListener listener]];
+  if ([listenerConnection registerName: name] == NO)
+    {
+      DESTROY(listenerConnection);
+    }
+
   if (listenerConnection != nil)
     {
       RETAIN(listenerConnection);
@@ -272,34 +278,39 @@ NSRegisterServicesProvider(id provider, NSString *name)
 
 - (void) dealloc
 {
+  GSNOSUPERDEALLOC;
 }
 
 /**
- * Selectively forwards those messages which are known to be safe.
+ * Selectively forwards those messages which are thought to be safe,
+ * and perform any special operations we need for workspace management
+ * etc.<br />
+ * The logic in this method <strong>must</strong> match that in 
+ * methodSignatureForSelector:
  */
 - (void) forwardInvocation: (NSInvocation*)anInvocation
 {
   SEL		aSel = [anInvocation selector];
   NSString      *selName = NSStringFromSelector(aSel);
+  id		target = nil;
+  id		delegate;
 
-  if ([selName isEqualToString: @"terminate:"])
+  /*
+   * We never permit remote processes to call private methods in this
+   * application.
+   */
+  if ([selName hasPrefix: @"_"] == YES)
     {
-      NSNotificationCenter	*c;
-
-      /*
-       * Send a power off notification before asking app to terminate.
-       */
-      c = [[NSWorkspace sharedWorkspace] notificationCenter];
-      [c _postLocal: NSWorkspaceWillPowerOffNotification userInfo: nil];
-
-      [anInvocation invokeWithTarget: NSApp];
-      return;
+      [NSException raise: NSGenericException
+		  format: @"method name '%@' private in '%@'",
+	selName, [manager port]];
     }
-  else if ([selName hasSuffix: @":userData:error:"])
+
+  if ([selName hasSuffix: @":userData:error:"] == YES)
     {
       /*
-       *    The selector matches the correct form for a services request,
-       *    so send the message to the services provider.
+       * The selector matches the correct form for a services request,
+       * so we send the message to the services provider.
        */
       if ([servicesProvider respondsToSelector: aSel] == YES)
 	{
@@ -315,59 +326,85 @@ NSRegisterServicesProvider(id provider, NSString *name)
 	  [anInvocation getArgument: (void*)&pb atIndex: 2];
 	  pb = [NSPasteboard pasteboardWithName: [pb name]];
 	  [anInvocation setArgument: (void*)&pb atIndex: 2];
-
 	  [anInvocation invokeWithTarget: servicesProvider];
 	  return;
 	}
+      [NSException raise: NSGenericException
+		  format: @"service request '%@' not implemented in '%@'",
+	selName, [manager port]];
+    }
+
+  delegate = [[NSApplication sharedApplication] delegate];
+
+  /*
+   * We assume that messages of the form 'application:...' are all
+   * safe and do mnot need to be listed in GSPermittedMessages.
+   * They can be handled either by the application delegate or by
+   * the shared GSServicesManager instance.
+   */
+  if ([selName hasPrefix: @"application:"] == YES)
+    {
+      if ([delegate respondsToSelector: aSel] == YES)
+	{
+	  target = delegate;
+	}
+      else if ([manager respondsToSelector: aSel] == YES)
+	{
+	  target = manager;
+	}
+    }
+
+  if (target == nil)
+    {
+      NSArray	*messages;
+ 
+      /*
+       * Unless the message was of a format assumed to be safe,
+       * we must check it against the GSPermittedMessages array
+       * to see if the app allows it to be sent from a remote
+       * process.
+       */
+      messages = [[NSUserDefaults standardUserDefaults] arrayForKey:
+	@"GSPermittedMessages"];
+      if (messages != nil && [messages containsObject: selName] == NO)
+	{
+	  [NSException raise: NSGenericException
+		      format: @"method '%@' not in GSPermittedMessages in '%@'",
+	    selName, [manager port]];
+	}
+      if ([delegate respondsToSelector: aSel] == YES)
+	{
+	  target = delegate;
+	}
+      else if ([NSApp respondsToSelector: aSel] == YES)
+	{
+	  target = NSApp;
+	}
+    }
+
+  if (target == nil)
+    {
+      [NSException raise: NSGenericException
+		  format: @"method '%@' not implemented in '%@'",
+	selName, [manager port]];
     }
   else
     {
-      id	delegate = [[NSApplication sharedApplication] delegate];
-
-      if ([selName isEqualToString: @"activateIgnoringOtherApps:"])
+      if ([selName isEqualToString: @"terminate:"])
 	{
-	  [anInvocation invokeWithTarget: NSApp];
-	  return;
+	  NSNotificationCenter	*c;
+
+	  /*
+	   * We handle the terminate: message as a special case, sending
+	   * a power off notification before allowing it to be processed
+	   * as normal.
+	   */
+	  c = [[NSWorkspace sharedWorkspace] notificationCenter];
+	  [c _postLocal: NSWorkspaceWillPowerOffNotification userInfo: nil];
 	}
 
-      if ([selName hasPrefix: @"application:"] == YES)
-	{
-	  if ([delegate respondsToSelector: aSel] == YES)
-	    {
-	      [anInvocation invokeWithTarget: delegate];
-	      return;
-	    }
-	  else if ([manager respondsToSelector: aSel] == YES)
-	    {
-	      [anInvocation invokeWithTarget: manager];
-	      return;
-	    }
-	}
-      else
-	{
-	  if ([delegate respondsToSelector: aSel] == YES)
-	    {
-	      NSArray	*messages;
-
-	      messages = [[NSUserDefaults standardUserDefaults] arrayForKey:
-		@"GSPermittedMessages"];
-	      if (messages != nil)
-		{
-		  if ([messages containsObject: selName] == YES)
-		    {
-		      [anInvocation invokeWithTarget: delegate];
-		    }
-		}
-	      else
-		{
-		  [anInvocation invokeWithTarget: delegate];
-		}
-	      return;
-	    }
-	}
+      [anInvocation invokeWithTarget: target];
     }
-  [NSException raise: NSGenericException
-	      format: @"method %@ not implemented", selName];
 }
 
 /**
@@ -375,54 +412,51 @@ NSRegisterServicesProvider(id provider, NSString *name)
  * to see if it's a standard service message or standard application
  * message.<br />
  * If the message is non-standard, it can be checked against a list
- * of messages specified by the GSPermittedMessages user default.
+ * of messages specified by the GSPermittedMessages user default.<br />
+ * The logic in this method <strong>must</strong> match that in 
+ * forwardInvocation:
  */
 - (NSMethodSignature*) methodSignatureForSelector: (SEL)aSelector
 {
   NSMethodSignature	*sig = nil;
   NSString      	*selName = NSStringFromSelector(aSelector);
+  id			delegate; 
 
-  if ([selName isEqualToString: @"terminate:"])
+  if ([selName hasSuffix: @":userData:error:"])
     {
-      sig = [NSApp methodSignatureForSelector: aSelector];
+      return [servicesProvider methodSignatureForSelector: aSelector];
     }
-  else if ([selName hasSuffix: @":userData:error:"])
-    {
-      sig = [servicesProvider methodSignatureForSelector: aSelector];
-    }
-  else
-    {
-      id	delegate = [[NSApplication sharedApplication] delegate];
 
-      if ([selName hasPrefix: @"application:"] == YES)
+  delegate = [[NSApplication sharedApplication] delegate];
+  if ([selName hasPrefix: @"application:"] == YES)
+    {
+      if ([delegate respondsToSelector: aSelector] == YES)
 	{
-	  if ([delegate respondsToSelector: aSelector] == YES)
-	    {
-	      sig = [delegate methodSignatureForSelector: aSelector];
-	    }
-	  else
-	    {
-	      sig = [manager methodSignatureForSelector: aSelector];
-	    }
+	  sig = [delegate methodSignatureForSelector: aSelector];
 	}
-      else
+      else if ([manager respondsToSelector: aSelector] == YES)
 	{
-	  NSArray	*messages;
+	  sig = [manager methodSignatureForSelector: aSelector];
+	}
+    }
 
-	  messages = [[NSUserDefaults standardUserDefaults] arrayForKey:
-	    @"GSPermittedMessages"];
+  if (sig == nil)
+    {
+      NSArray	*messages;
 
-	  if (messages != nil)
-	    {
-	      if ([messages containsObject: selName] == YES)
-		{
-		  sig = [delegate methodSignatureForSelector: aSelector];
-		}
-	    }
-	  else
-	    {
-	      sig = [delegate methodSignatureForSelector: aSelector];
-	    }
+      messages = [[NSUserDefaults standardUserDefaults] arrayForKey:
+	@"GSPermittedMessages"];
+      if (messages != nil && [messages containsObject: selName] == NO)
+	{
+	  return nil;
+	}
+      if ([delegate respondsToSelector: aSelector] == YES)
+	{
+	  sig = [delegate methodSignatureForSelector: aSelector];
+	}
+      else if ([NSApp respondsToSelector: aSelector] == YES)
+	{
+	  sig = [NSApp methodSignatureForSelector: aSelector];
 	}
     }
   return sig;
@@ -611,7 +645,7 @@ static NSString         *disabledName = @".GNUstepDisabled";
   unsigned      i, j;
   unsigned      es = [sendTypes count];
   unsigned      er = [returnTypes count];
-  NSWindow      *resp = [[_application keyWindow] firstResponder];
+  NSResponder   *resp = [[_application keyWindow] firstResponder];
   id            obj = nil;
 
   for (i = 0; i <= es; i++)
@@ -1234,7 +1268,7 @@ static NSString         *disabledName = @".GNUstepDisabled";
   unsigned      i, j;
   unsigned      es = [sendTypes count];
   unsigned      er = [returnTypes count];
-  NSWindow      *resp = [[_application keyWindow] firstResponder];
+  NSResponder	*resp = [[_application keyWindow] firstResponder];
 
   /*
    *    If the menu item is not in our map, it must be the item containing
