@@ -20,7 +20,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #include <AppKit/NSView.h>
@@ -356,7 +356,7 @@
 }
 
 // We do not want to popup menus in this menu.
-- (id) menuForEvent: (NSEvent*) theEvent
+- (NSMenu *) menuForEvent: (NSEvent*) theEvent
 {
   return nil;
 }
