@@ -285,11 +285,21 @@ static Class controlClass;
 - (void) encodeWithCoder: (NSCoder*)aCoder
 {
   [super encodeWithCoder: aCoder];
-  [aCoder encodeValueOfObjCType: @encode(int) at: &_tag];
-  [aCoder encodeConditionalObject: _target];
-  [aCoder encodeValueOfObjCType: @encode(SEL) at: &_action];
-  // This is only encoded for backward compatibility and won't be decoded.
-  [aCoder encodeConditionalObject: nil];
+  if([aCoder allowsKeyedCoding])
+    {
+      [aCoder encodeInt: [self tag] forKey: @"NSTag"];
+      [aCoder encodeObject: [self target] forKey: @"NSTarget"];
+      [aCoder encodeObject: NSStringFromSelector([self action]) forKey: @"NSAction"];
+      [aCoder encodeObject: _control_view forKey: @"NSControlView"];
+    }
+  else
+    {
+      [aCoder encodeValueOfObjCType: @encode(int) at: &_tag];
+      [aCoder encodeConditionalObject: _target];
+      [aCoder encodeValueOfObjCType: @encode(SEL) at: &_action];
+      // This is only encoded for backward compatibility and won't be decoded.
+      [aCoder encodeConditionalObject: nil];
+    }
 }
 
 - (id) initWithCoder: (NSCoder*)aDecoder
@@ -302,6 +312,15 @@ static Class controlClass;
         {
 	  [self setTag: [aDecoder decodeIntForKey: @"NSTag"]];
 	}
+      if ([aDecoder containsValueForKey: @"NSTarget"])
+	{
+	  [self setTarget: [aDecoder decodeObjectForKey: @"NSTarget"]];
+	}
+      if ([aDecoder containsValueForKey: @"NSAction"])
+        {
+	  NSString *action = [aDecoder decodeObjectForKey: @"NSAction"];
+	  [self setAction: NSSelectorFromString(action)];
+	}
     }
   else
     {
