@@ -36,6 +36,13 @@
 #include "AppKit/NSGraphics.h"
 
 #include <jerror.h>
+#if defined(__MINGW32__)
+/* Hack so that INT32 is not redefined in jmorecfg.h. MingW defines this
+   as well in basetsd.h */
+#ifndef XMD_H
+#define XMD_H
+#endif
+#endif
 #include <jpeglib.h>
 
 #include <setjmp.h>
@@ -263,7 +270,7 @@ nil. */
     {
       /* assign the description of possible occured error to errorMsg */
       if (errorMsg)
-	*errorMsg = (jerrMgr.error ? jerrMgr.error : nil);
+	*errorMsg = (jerrMgr.error ? (id)jerrMgr.error : (id)nil);
       gs_jpeg_memory_src_destroy(&cinfo);
       jpeg_destroy_decompress(&cinfo);
       if (imgbuffer)
