@@ -52,9 +52,57 @@
 #include "AppKit/NSSound.h"
 #include "AppKit/NSWindow.h"
 #include "GNUstepGUI/GSDrawFunctions.h"
+#include "GNUstepGUI/GSNibCompatibility.h"
 
 #include <math.h>
 
+typedef struct _GSButtonCellFlags 
+{
+#ifdef WORDS_BIGENDIAN
+  unsigned int isPushin:1;
+  unsigned int changeContents:1;
+  unsigned int changeBackground:1;
+  unsigned int changeGray:1;
+  unsigned int highlightByContents:1;
+  unsigned int highlightByBackground:1;
+  unsigned int highlightByGray:1;
+  unsigned int drawing:1;
+  unsigned int isBordered:1;
+  unsigned int imageDoesOverlap:1;
+  unsigned int isHorizontal:1;
+  unsigned int isBottomOrLeft:1;
+  unsigned int isImageAndText:1;
+  unsigned int isImageSizeDiff:1;
+  unsigned int hasKeyEquiv:1;
+  unsigned int lastState:1;
+  unsigned int isTransparent:1;
+  unsigned int unused1:6; // inset:2 doesn't dim:1 gradient:3
+  unsigned int useButtonImageSource:1;
+  unsigned int unused2:8; // alt mnemonic loc.
+#else
+  unsigned int unused2:8; // alt mnemonic loc.
+  unsigned int useButtonImageSource:1;
+  unsigned int unused1:6; // inset:2 doesn't dim:1 gradient:3
+  unsigned int isTransparent:1;
+  unsigned int lastState:1;
+  unsigned int hasKeyEquiv:1;
+  unsigned int isImageSizeDiff:1;
+  unsigned int isImageAndText:1;
+  unsigned int isBottomOrLeft:1;
+  unsigned int isHorizontal:1;
+  unsigned int imageDoesOverlap:1;
+  unsigned int isBordered:1;
+  unsigned int drawing:1;
+  unsigned int highlightByGray:1;
+  unsigned int highlightByBackground:1;
+  unsigned int highlightByContents:1;
+  unsigned int changeGray:1;
+  unsigned int changeBackground:1;
+  unsigned int changeContents:1;
+  unsigned int isPushin:1;
+#endif
+} GSButtonCellFlags;
+
 @interface NSButtonCell (Private)
 // Overriden private internal method
 - (void) _drawImage: (NSImage *)anImage inFrame: (NSRect)aRect 
@@ -426,7 +474,28 @@
     
   if (_cell.image_position == NSNoImage)
     {
-      [super setType: NSTextCellType];
+      /* NOTE: If we always call -setType: on superclass, the cell _content will
+         be reset each time. When we alter a button displaying both an image and
+         a title, by just calling -setImagePosition: with 'NSNoImage' value,
+         the current title is reset to NSCell default one. That's why we have to
+         set cell type ourself when a custom title (or attributed title) is
+         already in use. 
+         Take note that [self title] is able to return the attributed title in
+         NSString form.
+         We precisely match Mac OS X behavior currently. That means... When you
+         switch from 'NSNoImage' option to another one, the title will be the
+         one in use before you switched to 'NSNoImage'. The reverse with the
+         image isn't true, when you switch to 'NSNoImage' option, the current
+         image is lost (image value being reset to nil).
+      */
+      if ([self title] == nil || [[self title] isEqualToString: @""])
+        {
+          [super setType: NSTextCellType];
+        }
+      else
+        { 
+          _cell.type = NSTextCellType;
+        }
     }
   else
     {
@@ -1373,24 +1442,111 @@
  */
 - (void) encodeWithCoder: (NSCoder*)aCoder
 {
-  // FIXME: Add new ivars
   BOOL tmp;
+
   [super encodeWithCoder: aCoder];
+  if([aCoder allowsKeyedCoding])
+    {
+      GSButtonCellFlags buttonCellFlags;
+      unsigned int bFlags = 0;
+      unsigned int bFlags2 = 0;
+      NSImage *image = [self image];
+      NSButtonImageSource *bi = nil;
 
-  [aCoder encodeObject: _keyEquivalent];
-  [aCoder encodeObject: _keyEquivalentFont];
-  [aCoder encodeObject: _altContents];
-  [aCoder encodeObject: _altImage];
-  tmp = _buttoncell_is_transparent;
-  [aCoder encodeValueOfObjCType: @encode(BOOL)
-			     at: &tmp];
-  [aCoder encodeValueOfObjCType: @encode(unsigned int)
-			     at: &_keyEquivalentModifierMask];
-  [aCoder encodeValueOfObjCType: @encode(unsigned int)
-			     at: &_highlightsByMask];
-  [aCoder encodeValueOfObjCType: @encode(unsigned int)
-			     at: &_showAltStateMask];
+      if([self keyEquivalent] != nil)
+	{
+	  [aCoder encodeObject: [self keyEquivalent] forKey: @"NSKeyEquivalent"];
+	}
+      if([self image] != nil)
+	{
+	  [aCoder encodeObject: [self image] forKey: @"NSNormalImage"];
+	}
+      if([self alternateTitle] != nil)
+	{
+	  [aCoder encodeObject: [self alternateTitle] forKey: @"NSAlternateContents"];
+	}
 
+      buttonCellFlags.useButtonImageSource = (([NSImage imageNamed: @"NSSwitch"] == image) ||
+					      ([NSImage imageNamed: @"NSRadioButton"] == image));
+      buttonCellFlags.isTransparent = [self isTransparent];
+      buttonCellFlags.isBordered = [self isBordered]; 
+      buttonCellFlags.isImageAndText = (image != nil);
+      buttonCellFlags.hasKeyEquiv = ([self keyEquivalent] != nil);
+
+      // cell attributes...
+      buttonCellFlags.isPushin = [self cellAttribute: NSPushInCell]; 
+      buttonCellFlags.highlightByBackground = [self cellAttribute: NSCellLightsByBackground];
+      buttonCellFlags.highlightByContents = [self cellAttribute: NSCellLightsByContents];
+      buttonCellFlags.highlightByGray = [self cellAttribute: NSCellLightsByGray];
+      buttonCellFlags.changeBackground = [self cellAttribute: NSChangeBackgroundCell];
+      buttonCellFlags.changeContents = [self cellAttribute: NSCellChangesContents];
+      buttonCellFlags.changeGray = [self cellAttribute: NSChangeGrayCell];
+
+      // set these to zero...
+      buttonCellFlags.unused1 = 0; // 32;
+      buttonCellFlags.unused2 = 0; // 255;
+      buttonCellFlags.lastState = 0;
+      buttonCellFlags.isImageSizeDiff = 0;
+      buttonCellFlags.imageDoesOverlap = 0;
+      buttonCellFlags.drawing = 0;
+      buttonCellFlags.isBottomOrLeft = 0;
+
+      memcpy((void *)&bFlags, (void *)&buttonCellFlags,sizeof(unsigned int));
+      [aCoder encodeInt: bFlags forKey: @"NSButtonFlags"];
+
+      // style and border.
+      bFlags2 != [self showsBorderOnlyWhileMouseInside] ? 0x8 : 0;
+      bFlags2 |= [self bezelStyle];
+      [aCoder encodeInt: bFlags2 forKey: @"NSButtonFlags2"];
+
+      // alternate image encoding...
+      if(image != nil)
+	{
+	  if ([image isKindOfClass: [NSImage class]] && buttonCellFlags.useButtonImageSource)
+	    {
+	      if([NSImage imageNamed: @"NSSwitch"] == image)
+		{
+		  bi = [[NSButtonImageSource alloc] initWithImageNamed: @"NSHighlightedSwitch"];
+		}
+	      else if([NSImage imageNamed: @"NSRadioButton"] == image)
+		{
+		  bi = [[NSButtonImageSource alloc] initWithImageNamed: @"NSHighlightedRadioButton"];
+		}
+	    }
+	}
+
+      // encode button image source, if it exists...
+      if(bi != nil)
+	{
+	  [aCoder encodeObject: bi forKey: @"NSAlternateImage"];      
+	}
+      else if(_altImage != nil)
+	{
+	  [aCoder encodeObject: _altImage forKey: @"NSAlternateImage"];
+	}
+
+      // repeat and delay
+      [aCoder encodeInt: (int)_delayInterval forKey: @"NSPeriodicDelay"];
+      [aCoder encodeInt: (int)_repeatInterval forKey: @"NSPeriodicInterval"];
+    }
+  else
+    {
+      // FIXME: Add new ivars
+      [aCoder encodeObject: _keyEquivalent];
+      [aCoder encodeObject: _keyEquivalentFont];
+      [aCoder encodeObject: _altContents];
+      [aCoder encodeObject: _altImage];
+      tmp = _buttoncell_is_transparent;
+      [aCoder encodeValueOfObjCType: @encode(BOOL)
+	      at: &tmp];
+      [aCoder encodeValueOfObjCType: @encode(unsigned int)
+	      at: &_keyEquivalentModifierMask];
+      [aCoder encodeValueOfObjCType: @encode(unsigned int)
+	      at: &_highlightsByMask];
+      [aCoder encodeValueOfObjCType: @encode(unsigned int)
+	      at: &_showAltStateMask];
+      
+    }
 }
 
 - (id) initWithCoder: (NSCoder*)aDecoder
@@ -1399,10 +1555,10 @@
 
   if ([aDecoder allowsKeyedCoding])
     {
-      //NSControl *control = [aDecoder decodeObjectForKey: @"NSControlView"];
       int delay = 0;
-      int interval = 0;
-      
+      int interval = 0;      
+      // NSControl *control = [aDecoder decodeObjectForKey: @"NSControlView"];
+
       if ([aDecoder containsValueForKey: @"NSKeyEquivalent"])
         {
 	  [self setKeyEquivalent: [aDecoder decodeObjectForKey: @"NSKeyEquivalent"]];
@@ -1411,57 +1567,34 @@
         {
 	  [self setImage: [aDecoder decodeObjectForKey: @"NSNormalImage"]];
 	}
-      if ([aDecoder containsValueForKey: @"NSAlternateImage"])
-        {
-	  id image;
-	  
-	  image = [aDecoder decodeObjectForKey: @"NSAlternateImage"];
-	  // This test works around an Apple bug, where a font gets encoded here. 
-	  if ([image isKindOfClass: [NSImage class]])
-	    {
-	      [self setAlternateImage: image];
-	    }
-	}
       if ([aDecoder containsValueForKey: @"NSAlternateContents"])
         {
 	  [self setAlternateTitle: [aDecoder decodeObjectForKey: @"NSAlternateContents"]];
 	}
       if ([aDecoder containsValueForKey: @"NSButtonFlags"])
         {
-	  int bFlags;
-	  int highlights = 0;  
-	  int show_state = NSNoCellMask;  
+	  unsigned int bFlags = [aDecoder decodeIntForKey: @"NSButtonFlags"];
+	  GSButtonCellFlags buttonCellFlags;
+	  memcpy((void *)&buttonCellFlags,(void *)&bFlags,sizeof(struct _GSButtonCellFlags));
 
-	  bFlags = [aDecoder decodeIntForKey: @"NSButtonFlags"];
-
-	  [self setTransparent: (bFlags & 0x8000)];
-	  // FIXME
-	  if ((bFlags & 0x800000) == 0x800000)
-	    {
-	      [self setBordered: YES];
-	    }
-	  if ((bFlags & 0x6000000) == 0x6000000)
-	    {
-	      highlights |= NSChangeBackgroundCellMask;
-	    }
-	  if ((bFlags & 0x8000000) == 0x8000000)
-	    {
-	      highlights |= NSContentsCellMask;
-	    }
-	  if ((bFlags & 0x30000000) == 0x30000000)
-	    {
-	      show_state |= NSChangeBackgroundCellMask;
-	    }
-	  if ((bFlags & 0x40000000) == 0x40000000)
-	    {
-	      show_state |= NSContentsCellMask;
-	    }
-	  if ((bFlags & 0x80000000) == 0x80000000)
-	    {
-	      highlights |= NSPushInCellMask;
-	    }
-	  [self setHighlightsBy: highlights];
-	  [self setShowsStateBy: show_state];
+	  [self setTransparent: buttonCellFlags.isTransparent];
+	  [self setBordered: buttonCellFlags.isBordered];
+	  
+	  [self setCellAttribute: NSPushInCell
+		to: buttonCellFlags.isPushin];
+	  [self setCellAttribute: NSCellLightsByBackground 
+		to: buttonCellFlags.highlightByBackground];
+	  [self setCellAttribute: NSCellLightsByContents   
+		to: buttonCellFlags.highlightByContents];
+	  [self setCellAttribute: NSCellLightsByGray
+		to: buttonCellFlags.highlightByGray]; 
+	  [self setCellAttribute: NSChangeBackgroundCell   
+		to: buttonCellFlags.changeBackground];
+	  [self setCellAttribute: NSCellChangesContents    
+		to: buttonCellFlags.changeContents];
+	  [self setCellAttribute: NSChangeGrayCell
+		to: buttonCellFlags.changeGray]; 
+	  
 	  [self setImagePosition: NSImageLeft];
 	}
       if ([aDecoder containsValueForKey: @"NSButtonFlags2"])
@@ -1507,6 +1640,40 @@
 		break;
 	    }
 	}
+      if ([aDecoder containsValueForKey: @"NSAlternateImage"])
+        {
+	  id image;
+
+	  //
+	  // NOTE: Okay... this is a humongous kludge.   It seems as though
+	  // Cocoa is doing something very odd here.  It doesn't seem to 
+	  // encode system images for buttons normally, if it is using 
+	  // images at all. Until I figure out what, this will stay.  
+	  // Danger, Will Robinson! :)
+	  //
+	  image = [aDecoder decodeObjectForKey: @"NSAlternateImage"];
+	  if ([image isKindOfClass: [NSImage class]])
+	    {
+	      if([NSImage imageNamed: @"NSSwitch"] == image)
+		{
+		  image = [NSImage imageNamed: @"NSHighlightedSwitch"];
+		  if([self image] == nil)
+		    {
+		      [self setImage: [NSImage imageNamed: @"NSSwitch"]];
+		    }		    
+		}
+	      else if([NSImage imageNamed: @"NSRadioButton"] == image)
+		{
+		  image = [NSImage imageNamed: @"NSHighlightedRadioButton"];
+		  if([self image] == nil)
+		    {
+		      [self setImage: [NSImage imageNamed: @"NSRadioButton"]];
+		    }		    
+		}
+	      
+	      [self setAlternateImage: image];
+	    }
+	}
       if ([aDecoder containsValueForKey: @"NSPeriodicDelay"])
         {
 	  delay = [aDecoder decodeIntForKey: @"NSPeriodicDelay"];
