@@ -688,30 +688,24 @@ static NSColor	*shadowCol;
 }
 
 /**<p>Sets whether the cell has a bezeled border. 
- If the cell has a bezeled border, the bordered  flag is turn off.
+ If this method is called, the bordered flag is turn off.
  By default a NSCell has no bezeled border</p>
  <p>See Also: -isBezeled -setBordered: -isBordered</p>
  */
 - (void) setBezeled: (BOOL)flag
 {
   _cell.is_bezeled = flag;
-  if (_cell.is_bezeled)
-    {
-      _cell.is_bordered = NO;
-    }
+  _cell.is_bordered = NO;
 }
 
-/**<p>Sets whether the cell has a border.  If the cell has a border, 
+/**<p>Sets whether the cell has a border.  If this method is called, 
  the bezeled flag is turn off. By default a NSCell has no border</p>
  <p>See Also: -isBordered -setBezeled: -isBezeled</p>
  */
 - (void) setBordered: (BOOL)flag
 {
   _cell.is_bordered = flag;
-  if (_cell.is_bordered)
-    {
-      _cell.is_bezeled = NO;
-    }
+  _cell.is_bezeled = NO;
 }
 
 /**<p>Sets the NSCell's state.  Please use always symbolic constants when
@@ -1528,6 +1522,7 @@ static NSColor	*shadowCol;
   NSPoint	last_point = point;
   BOOL		done;
   BOOL		mouseWentUp;
+  unsigned	periodCount = 0;
 
   NSDebugLLog(@"NSCell", @"cell start tracking in rect %@ initial point %f %f",
 	     NSStringFromRect(cellFrame), point.x, point.y);
@@ -1557,7 +1552,6 @@ static NSColor	*shadowCol;
     {
       NSEventType	eventType;
       BOOL		pointIsInCell;
-      unsigned		periodCount = 0;
 
       theEvent = [theApp nextEventMatchingMask: event_mask
 				     untilDate: nil
@@ -2185,84 +2179,140 @@ static NSColor	*shadowCol;
  */
 - (void) encodeWithCoder: (NSCoder*)aCoder
 {
-  BOOL flag;
-  unsigned int tmp_int;
+  if([aCoder allowsKeyedCoding])
+    {
+      unsigned long cFlags = 0;
+      unsigned int cFlags2 = 0;
 
-  [aCoder encodeObject: _contents];
-  [aCoder encodeObject: _cell_image];
-  [aCoder encodeObject: _font];
-  [aCoder encodeObject: _objectValue];
-  flag = _cell.contents_is_attributed_string;
-  [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
-  flag = _cell.is_highlighted;
-  [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
-  flag = _cell.is_disabled;
-  [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
-  flag = _cell.is_editable;
-  [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
-  flag = _cell.is_rich_text;
-  [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
-  flag = _cell.imports_graphics;
-  [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
-  flag = _cell.shows_first_responder;
-  [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
-  flag = _cell.refuses_first_responder;
-  [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
-  flag = _cell.sends_action_on_end_editing;
-  [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
-  flag = _cell.is_bordered;
-  [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
-  flag = _cell.is_bezeled;
-  [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
-  flag = _cell.is_scrollable;
-  [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
-  flag = _cell.is_selectable;
-  [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
-  // This used to be is_continuous, which has been replaced.
-  /* Ayers 20.03.2003: But we must continue to encode it for backward
-     compatibility or current releases will have undefined behavior when
-     decoding archives (i.e. .gorm files) encoded by this version. */
-  flag = [self isContinuous];
-  [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
-  flag = _cell.allows_mixed_state;
-  [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
-  flag = _cell.wraps;
-  [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
-  tmp_int = _cell.text_align;
-  [aCoder encodeValueOfObjCType: @encode(unsigned int) at: &tmp_int];
-  tmp_int = _cell.type;
-  [aCoder encodeValueOfObjCType: @encode(unsigned int) at: &tmp_int];
-  tmp_int = _cell.image_position;
-  [aCoder encodeValueOfObjCType: @encode(unsigned int) at: &tmp_int];
-  tmp_int = _cell.entry_type;
-  [aCoder encodeValueOfObjCType: @encode(unsigned int) at: &tmp_int];
-  tmp_int = _cell.state;
-  [aCoder encodeValueOfObjCType: @encode(unsigned int) at: &tmp_int];
-  [aCoder encodeValueOfObjCType: @encode(unsigned int) at: &_mnemonic_location];
-  [aCoder encodeValueOfObjCType: @encode(unsigned int) at: &_mouse_down_flags];
-  [aCoder encodeValueOfObjCType: @encode(unsigned int) at: &_action_mask];
-  [aCoder encodeValueOfObjCType: @encode(id) at: &_formatter];
-  [aCoder encodeValueOfObjCType: @encode(id) at: &_menu];
-  [aCoder encodeValueOfObjCType: @encode(id) at: &_represented_object];
+      // encode contents
+      [aCoder encodeObject: _contents forKey: @"NSContents"];
+
+      // flags
+      cFlags |= [self wraps] ?  0x40 : 0;
+      cFlags |= [self isScrollable] ? 0x100000 : 0; 
+      cFlags |= [self isSelectable] ? 0x200001 : 0;
+      cFlags |= [self isBezeled] ? 0x400000 : 0;
+      cFlags |= [self isBordered] ? 0x800000 : 0;
+      cFlags |= ([self type] == NSTextCellType) ? 0x4000000 : 0;
+      cFlags |= [self isContinuous] ? 0x40000 : 0;
+      cFlags |= [self isEditable] ? 0x10000000 : 0;
+      cFlags |= ([self isEnabled] == NO) ? 0x20000000 : 0;
+      cFlags |= ([self state] == NSOnState) ? 0x80000000 : 0;
+      cFlags |= [self isHighlighted] ? 0x40000000 : 0;
+      [aCoder encodeInt: cFlags forKey: @"NSCellFlags"];
+      
+      // flags part 2
+      cFlags2 |= [self sendsActionOnEndEditing] ? 0x400000 : 0;
+      cFlags2 |= [self allowsMixedState] ? 0x1000000 : 0;
+      cFlags2 |= [self refusesFirstResponder] ? 0x2000000 : 0;
+      cFlags2 |= ([self alignment] == NSRightTextAlignment) ? 0x4000000 : 0;
+      cFlags2 |= ([self alignment] == NSCenterTextAlignment) ? 0x8000000 : 0;
+      cFlags2 |= ([self alignment] == NSJustifiedTextAlignment) ? 0xC000000 : 0;
+      cFlags2 |= ([self alignment] == NSNaturalTextAlignment) ? 0x10000000 : 0;
+      cFlags2 |= [self importsGraphics] ? 0x20000000 : 0;
+      cFlags2 |= [self allowsEditingTextAttributes] ? 0x40000000 : 0;
+      [aCoder encodeInt: cFlags2 forKey: @"NSCellFlags2"];
+	  
+      // font and formatter.
+      [aCoder encodeObject: [self font] forKey: @"NSSupport"];
+
+      if([self formatter])
+	{
+	  [aCoder encodeObject: [self formatter] forKey: @"NSFormatter"];
+	}
+    }
+  else
+    {
+      BOOL flag;
+      unsigned int tmp_int;
+      
+      [aCoder encodeObject: _contents];
+      [aCoder encodeObject: _cell_image];
+      [aCoder encodeObject: _font];
+      [aCoder encodeObject: _objectValue];
+      flag = _cell.contents_is_attributed_string;
+      [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      flag = _cell.is_highlighted;
+      [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      flag = _cell.is_disabled;
+      [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      flag = _cell.is_editable;
+      [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      flag = _cell.is_rich_text;
+      [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      flag = _cell.imports_graphics;
+      [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      flag = _cell.shows_first_responder;
+      [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      flag = _cell.refuses_first_responder;
+      [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      flag = _cell.sends_action_on_end_editing;
+      [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      flag = _cell.is_bordered;
+      [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      flag = _cell.is_bezeled;
+      [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      flag = _cell.is_scrollable;
+      [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      flag = _cell.is_selectable;
+      [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      // This used to be is_continuous, which has been replaced.
+      /* Ayers 20.03.2003: But we must continue to encode it for backward
+	 compatibility or current releases will have undefined behavior when
+	 decoding archives (i.e. .gorm files) encoded by this version. */
+      flag = [self isContinuous];
+      [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      flag = _cell.allows_mixed_state;
+      [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      flag = _cell.wraps;
+      [aCoder encodeValueOfObjCType: @encode(BOOL) at: &flag];
+      tmp_int = _cell.text_align;
+      [aCoder encodeValueOfObjCType: @encode(unsigned int) at: &tmp_int];
+      tmp_int = _cell.type;
+      [aCoder encodeValueOfObjCType: @encode(unsigned int) at: &tmp_int];
+      tmp_int = _cell.image_position;
+      [aCoder encodeValueOfObjCType: @encode(unsigned int) at: &tmp_int];
+      tmp_int = _cell.entry_type;
+      [aCoder encodeValueOfObjCType: @encode(unsigned int) at: &tmp_int];
+      tmp_int = _cell.state;
+      [aCoder encodeValueOfObjCType: @encode(unsigned int) at: &tmp_int];
+      [aCoder encodeValueOfObjCType: @encode(unsigned int) at: &_mnemonic_location];
+      [aCoder encodeValueOfObjCType: @encode(unsigned int) at: &_mouse_down_flags];
+      [aCoder encodeValueOfObjCType: @encode(unsigned int) at: &_action_mask];
+      [aCoder encodeValueOfObjCType: @encode(id) at: &_formatter];
+      [aCoder encodeValueOfObjCType: @encode(id) at: &_menu];
+      [aCoder encodeValueOfObjCType: @encode(id) at: &_represented_object];
+    }
 }
 
 - (id) initWithCoder: (NSCoder*)aDecoder
 {
   if ([aDecoder allowsKeyedCoding])
     {
-      NSString *contents = [aDecoder decodeObjectForKey: @"NSContents"];
-
-      self = [self initTextCell: contents];
+      id contents = [aDecoder decodeObjectForKey: @"NSContents"];
 
+      // initialize based on content...
+      if([contents isKindOfClass: [NSString class]])
+	{
+	  self = [self initTextCell: contents];
+	}
+      else if([contents isKindOfClass: [NSImage class]])
+	{
+	  self = [self initImageCell: contents];
+	}
+      else
+	{
+	  self = [self init];
+	}
+      
       if ([aDecoder containsValueForKey: @"NSCellFlags"])
         {
-	  int cFlags;
+	  unsigned long cFlags;
 
 	  cFlags = [aDecoder decodeIntForKey: @"NSCellFlags"];
 
 	  // This bit flag is the other way around!
 	  [self setWraps: ((cFlags & 0x40) != 0x40)];
-	  [self setContinuous: ((cFlags & 0x80000) == 0x80000)];
 	  [self setScrollable: ((cFlags & 0x100000) == 0x100000)];
 	  // Strange that this is not a simple bit flag
 	  [self setSelectable: ((cFlags & 0x200001) == 0x200001)];
@@ -2272,12 +2322,13 @@ static NSColor	*shadowCol;
 	    {
 	      [self setType: NSTextCellType];
 	    }
+
+	  [self setContinuous: ((cFlags & 0x40000) == 0x40000)];
 	  [self setEditable: ((cFlags & 0x10000000) == 0x10000000)];
 	  // This bit flag is the other way around!
 	  [self setEnabled: ((cFlags & 0x20000000) != 0x20000000)];
 	  [self setHighlighted: ((cFlags & 0x40000000) == 0x40000000)];
-	  // FIXME
-	  
+	  [self setState: ((cFlags & 0x80000000) == 0x80000000)?NSOnState:NSOffState];
 	}
       if ([aDecoder containsValueForKey: @"NSCellFlags2"])
         {
@@ -2306,7 +2357,6 @@ static NSColor	*shadowCol;
 
 	  [self setImportsGraphics: ((cFlags2 & 0x20000000) == 0x20000000)];
 	  [self setAllowsEditingTextAttributes: ((cFlags2 & 0x40000000) == 0x40000000)];
-	  // FIXME
 	}
       if ([aDecoder containsValueForKey: @"NSSupport"])
         {
