@@ -24,7 +24,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */ 
 
 #include "config.h"
@@ -443,9 +443,17 @@ static NSLock *_gnustep_color_list_lock = nil;
 
   if (path == nil || ([fm fileExistsAtPath: path isDirectory: &isDir] == NO))
     {
+      NSArray	*paths;
+
       // FIXME the standard path for saving color lists?
-      path = [[NSSearchPathForDirectoriesInDomains(NSLibraryDirectory,
-	NSUserDomainMask, YES) objectAtIndex: 0]
+      paths = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory,
+	NSUserDomainMask, YES);
+      if ([paths count] == 0)
+	{
+	  NSLog (@"Failed to find Library directory for user");
+	  return NO;	// No directory to save to.
+	}
+      path = [[paths objectAtIndex: 0]
 	stringByAppendingPathComponent: @"Colors"]; 
       isDir = YES;
     }
@@ -489,14 +497,14 @@ static NSLock *_gnustep_color_list_lock = nil;
    */
   if (path_is_standard && ([fm fileExistsAtPath: path] == NO))
     {
-      if([fm createDirectoryAtPath: path 
+      if ([fm createDirectoryAtPath: path 
 			attributes: nil])
 	{
 	  NSLog (@"Created standard directory %@", path);
 	}
       else
 	{
-	  NSLog (@"Failed attemp to create directory %@", path);
+	  NSLog (@"Failed attempt to create directory %@", path);
 	}
     }
 
