@@ -695,12 +695,12 @@ static GSComboWindow *gsWindow = nil;
 {
   if (_cell != nil)
     {
-      NSText *textObject = nil;
-      NSControl *cv = [_cell controlView];
+      NSText	*textObject = nil;
+      id	cv = [_cell controlView];
       
       if ([cv isKindOfClass: [NSControl class]])
         {
-	   textObject = [(NSControl *)cv currentEditor];
+	  textObject = [(NSControl *)cv currentEditor];
 	}
       
       [_cell setStringValue: [_cell _stringValueAtIndex: 
@@ -711,13 +711,6 @@ static GSComboWindow *gsWindow = nil;
       // no effect, to correct this fact, the code below is needed.
       [textObject setString: [_cell _stringValueAtIndex:
         [_cell indexOfSelectedItem]]];
-      
-      /* 
-       * Dispatch the text notifications and by side effect update the cell 
-       * object value with the -textDidChange: method of NSTextField which is
-       * the editor delegate
-       */
-      [(NSTextView *)textObject didChangeText];
       // End of the code to remove 
       
       if  (textObject != nil)
@@ -1661,18 +1654,26 @@ static inline NSRect buttonCellFrameFromRect(NSRect cellRect)
 {
   [super encodeWithCoder: coder];
 
-  [coder encodeValueOfObjCType: @encode(id) at: &_popUpList];
-  [coder encodeValueOfObjCType: @encode(BOOL) at: &_usesDataSource];
-  [coder encodeValueOfObjCType: @encode(BOOL) at: &_hasVerticalScroller];
-  [coder encodeValueOfObjCType: @encode(BOOL) at: &_completes];
-  [coder encodeValueOfObjCType: @encode(BOOL) at: &_usesDataSource];
-  [coder encodeValueOfObjCType: @encode(int) at: &_visibleItems];
-  [coder encodeValueOfObjCType: @encode(NSSize) at: &_intercellSpacing];
-  [coder encodeValueOfObjCType: @encode(float) at: &_itemHeight];
-  [coder encodeValueOfObjCType: @encode(int) at: &_selectedItem];
-
-  if (_usesDataSource == YES)
-    [coder encodeConditionalObject: _dataSource];      
+  if([coder allowsKeyedCoding])
+    {
+      [coder encodeBool: [self hasVerticalScroller] forKey: @"NSHasVerticalScroller"];
+      [coder encodeInt: [self numberOfVisibleItems] forKey: @"NSVisibleItemCount"];
+    }
+  else
+    {
+      [coder encodeValueOfObjCType: @encode(id) at: &_popUpList];
+      [coder encodeValueOfObjCType: @encode(BOOL) at: &_usesDataSource];
+      [coder encodeValueOfObjCType: @encode(BOOL) at: &_hasVerticalScroller];
+      [coder encodeValueOfObjCType: @encode(BOOL) at: &_completes];
+      [coder encodeValueOfObjCType: @encode(BOOL) at: &_usesDataSource];
+      [coder encodeValueOfObjCType: @encode(int) at: &_visibleItems];
+      [coder encodeValueOfObjCType: @encode(NSSize) at: &_intercellSpacing];
+      [coder encodeValueOfObjCType: @encode(float) at: &_itemHeight];
+      [coder encodeValueOfObjCType: @encode(int) at: &_selectedItem];
+      
+      if (_usesDataSource == YES)
+	[coder encodeConditionalObject: _dataSource];      
+    }
 }
 
 /**
