@@ -21,7 +21,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */ 
 
 #include "config.h"
@@ -47,7 +47,7 @@ static NSDataLinkPanel *_sharedDataLinkPanel;
   NSString *panelPath;
   NSDictionary *table;
 
-  if((self = [super init]) != nil)
+  if ((self = [super init]) != nil)
     {
       panelPath = [GSGuiBundle() pathForResource: @"GSDataLinkPanel" 
 			      ofType: @"gorm"
@@ -113,7 +113,7 @@ static NSDataLinkPanel *_sharedDataLinkPanel;
  */
 + (NSDataLinkPanel *)sharedDataLinkPanel
 {
-  if(_sharedDataLinkPanel == nil)
+  if (_sharedDataLinkPanel == nil)
     {
       id controller = [[GSDataLinkPanelController alloc] init];
       _sharedDataLinkPanel = [controller panel];
