@@ -23,7 +23,7 @@
    
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
 */
    
 
@@ -770,6 +770,11 @@ NSGraphicsContext	*GSCurrentContext(void)
   [self subclassResponsibility: _cmd];
 }
 
+- (void) GSSetPatterColor: (NSImage*)image 
+{
+  [self subclassResponsibility: _cmd];
+}
+
 /**
    <p>Sets the colorspace for fill operations based on values in the supplied
    dictionary dict.</p>
@@ -1346,7 +1351,7 @@ NSGraphicsContext	*GSCurrentContext(void)
   DPSsetdash(self, pattern, count, phase);
 
   count = [path elementCount];
-  for(i = 0; i < count; i++) 
+  for (i = 0; i < count; i++) 
     {
       type = [path elementAtIndex: i associatedPoints: pts];
       switch(type) 
@@ -1438,6 +1443,19 @@ NSGraphicsContext	*GSCurrentContext(void)
   [self subclassResponsibility: _cmd];
 }
 
+/*
+  As currently not all backends support mixed composite and dissolve operations, 
+  this method is here to dispatch to the best suited one implemented
+ */
+- (void) GScomposite: (int)gstateNum
+	     toPoint: (NSPoint)aPoint
+	    fromRect: (NSRect)srcRect
+	   operation: (NSCompositingOperation)op
+	    fraction: (float)delta
+{
+  [self subclassResponsibility: _cmd];
+}
+
 /** Generic method to draw an image into a rect. The image is defined
     by imageref, an opaque structure. Support for this method hasn't
     been implemented yet, so it should not be used anywhere. */
