@@ -20,7 +20,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */ 
 
 #include "config.h"
@@ -181,7 +181,7 @@ NSInterfaceStyleForKey(NSString *key, NSResponder *responder)
   NSUserDefaults	*defs;
   NSMapEnumerator	enumerator;
   NSString		*key;
-  NSInterfaceStyle	val;
+  void                  *val;
 
   defs = [NSUserDefaults standardUserDefaults];
 
@@ -215,7 +215,7 @@ NSInterfaceStyleForKey(NSString *key, NSResponder *responder)
 	    }
 	}
 
-      if (newStyle != val)
+      if (newStyle != ((NSInterfaceStyle)val))
 	{
 	  NSMapInsert(styleMap, (void*)key, (void*)newStyle);
 	}
