@@ -1578,12 +1578,12 @@ dictionary.
 -(BOOL) layoutManagerOwnsFirstResponderInWindow: (NSWindow *)window
 {
   int i;
-  NSView *tv;
-  NSView *v = [window firstResponder];
+  NSResponder *tv;
+  NSResponder *v = [window firstResponder];
 
   for (i = 0; i < num_textcontainers; i++)
     {
-      tv = (NSView *)[textcontainers[i].textContainer textView];
+      tv = [textcontainers[i].textContainer textView];
       if (tv == v)
         return YES;
     }
@@ -2183,7 +2183,9 @@ no_soft_invalidation:
       if ([aDecoder containsValueForKey: @"NSLMFlags"])
         {
 	  flags = [aDecoder decodeIntForKey: @"NSLMFlags"];
-	  // FIXME
+	  // nothing really to do with these flags....
+	  // they are runtime flags which, even if set into the archive
+	  // they are ignored...
 	}
       [self setDelegate: delegate];
       [storage addLayoutManager: self];
