@@ -20,7 +20,7 @@
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
 */
 
 #include <math.h>
@@ -466,7 +466,7 @@ static NSMutableDictionary *units = nil;
   /* test markers in reverse order so that markers drawn on top
      are tested before those underneath */
   markerEnum = [_markers reverseObjectEnumerator];
-  while ( (marker = [markerEnum nextObject]) != nil)
+  while ((marker = [markerEnum nextObject]) != nil)
     {
       if (NSMouseInRect (point, [marker imageRectInRuler], flipped))
         {
@@ -753,7 +753,7 @@ static NSMutableDictionary *units = nil;
         }
       else
         {
-	  labelPosition.x = baselineLocation + LABEL_MARK_SIZE + 4 - size.width;
+	  labelPosition.x = baselineLocation + _ruleThickness - size.width;
 	  labelPosition.y = labelLocation + 1;
         }
       [labelString drawAtPoint: labelPosition withAttributes: attr];
@@ -872,7 +872,6 @@ static NSMutableDictionary *units = nil;
 - (void) dealloc
 {
   RELEASE(_unit);
-  RELEASE(_clientView);
   RELEASE(_accessoryView);
   RELEASE(_markers);
   RELEASE(_labelFormat);
