@@ -95,7 +95,7 @@ static BOOL _gs_display_reading_progress = NO;
 
 @implementation NSSavePanel (_PrivateMethods)
 
-- (unsigned int) draggingEntered: (id <NSDraggingInfo>)sender
+- (NSDragOperation) draggingEntered: (id <NSDraggingInfo>)sender
 {
   NSPasteboard *pb;
 
@@ -1067,6 +1067,7 @@ selectCellWithString: (NSString*)title
 {
   ASSIGN(_directory, [_browser pathToColumn:[_browser lastColumn]]);
   [NSApp stopModalWithCode: NSCancelButton];
+  [_okButton setEnabled: NO];
   [self close];
 }
 
@@ -1179,6 +1180,7 @@ selectCellWithString: (NSString*)title
       return;
 
   [NSApp stopModalWithCode: NSOKButton];
+  [_okButton setEnabled: NO];
   [self close];
 }
 
