@@ -30,62 +30,6 @@
 #include "AppKit/NSSlider.h"
 #include "AppKit/NSSliderCell.h"
 #include "AppKit/NSWindow.h"
-#include "AppKit/NSApplication.h"
-
-static inline
-float _floatValueForMousePoint (NSPoint point, NSRect knobRect,
-				NSRect slotRect, BOOL isVertical, 
-				float minValue, float maxValue, 
-				NSSliderCell *theCell, BOOL flipped)
-{
-  float floatValue = 0;
-  float position;
-
-  // Adjust the point to lie inside the knob slot. We don't
-  // have to worry whether the view is flipped or not.
-  if (isVertical)
-    {
-      if (point.y < slotRect.origin.y + knobRect.size.height / 2)
-	{
-	  position = slotRect.origin.y + knobRect.size.height / 2;
-	}
-      else if (point.y > slotRect.origin.y + slotRect.size.height
-	- knobRect.size.height / 2)
-	{
-	  position = slotRect.origin.y + slotRect.size.height
-	    - knobRect.size.height / 2;
-	}
-      else
-	position = point.y;
-										      // Compute the float value
-      floatValue = (position - (slotRect.origin.y + knobRect.size.height/2))
-	/ (slotRect.size.height - knobRect.size.height);
-      if (flipped)
-	floatValue = 1 - floatValue;
-    }
-  else
-    {
-      if (point.x < slotRect.origin.x + knobRect.size.width / 2)
-	{
-	  position = slotRect.origin.x + knobRect.size.width / 2;
-	}
-      else if (point.x > slotRect.origin.x + slotRect.size.width
-	- knobRect.size.width / 2)
-	{
-	  position = slotRect.origin.x + slotRect.size.width
-	    - knobRect.size.width / 2;
-	}
-      else
-	position = point.x;
-
-      // Compute the float value given the knob size
-      floatValue = (position - (slotRect.origin.x + knobRect.size.width / 2))
-	/ (slotRect.size.width - knobRect.size.width);
-    }
-
-  return floatValue * (maxValue - minValue) + minValue;
-}
-
 
 /**
   <unit>
@@ -361,109 +305,4 @@ static Class cellClass;
   return [_cell tickMarkValueAtIndex: index];
 }
 
-
-- (void) trackKnob: (NSEvent*)theEvent knobRect: (NSRect)knobRect
-{
-  NSApplication *app = [NSApplication sharedApplication];
-  unsigned int eventMask = NSLeftMouseDownMask | NSLeftMouseUpMask
-			  | NSLeftMouseDraggedMask | NSMouseMovedMask
-			  | NSPeriodicMask;
-  NSPoint point = [self convertPoint: [theEvent locationInWindow] 
-			fromView: nil];
-  NSEventType eventType = [theEvent type];
-  BOOL isContinuous = [_cell isContinuous];
-  float oldFloatValue = [_cell floatValue];
-  id target = [_cell target];
-  SEL action = [_cell action];
-  NSDate *distantFuture = [NSDate distantFuture];
-  NSRect slotRect = [_cell trackRect];
-  BOOL isVertical = [_cell isVertical];
-  float minValue = [_cell minValue];
-  float maxValue = [_cell maxValue];
-
-  [NSEvent startPeriodicEventsAfterDelay: 0.05 withPeriod: 0.05];
-  [[NSRunLoop currentRunLoop] limitDateForMode: NSEventTrackingRunLoopMode];
-
-  [self lockFocus];
-
-  while (eventType != NSLeftMouseUp)
-    {
-      theEvent = [app nextEventMatchingMask: eventMask
-				  untilDate: distantFuture
-				     inMode: NSEventTrackingRunLoopMode
-				    dequeue: YES];
-      eventType = [theEvent type];
-
-      if (eventType != NSPeriodic)
-	{
-	  point = [self convertPoint: [theEvent locationInWindow]
-			fromView: nil];
-	}
-      else
-	{
-	  if (point.x != knobRect.origin.x || point.y != knobRect.origin.y)
-	    {
-	      float floatValue;
-	      floatValue = _floatValueForMousePoint (point, knobRect,
-						     slotRect, isVertical, 
-						     minValue, maxValue, 
-						     _cell, 
-						     _rFlags.flipped_view); 
-	      if (floatValue != oldFloatValue)
-		{
-		  [_cell setFloatValue: floatValue];
-		  [_cell drawWithFrame: _bounds inView: self];
-		  [_window flushWindow];
-		  if (isContinuous)
-		    {
-		      [self sendAction: action to: target];
-		    }
-		  oldFloatValue = floatValue;
-		}
-	      knobRect.origin = point;
-	    }
-	}
-    }
-  [self unlockFocus];
-  // If the control is not continuous send the action at the end of the drag
-  if (!isContinuous)
-    {
-      [self sendAction: action to: target];
-    }
-  [NSEvent stopPeriodicEvents];
-}
-
-- (void) mouseDown: (NSEvent *)theEvent
-{
-  if ([_cell isEnabled])
-    {
-      NSPoint location = [self convertPoint: [theEvent locationInWindow] 
-			       fromView: nil];
-      NSRect rect;
-
-      rect = [_cell knobRectFlipped: _rFlags.flipped_view];
-      if (![self mouse: location inRect: rect])
-	{
-	  // Mouse is not on the knob, move the knob to the mouse position
-	  float floatValue;
-	  floatValue = _floatValueForMousePoint (location, rect, 
-						 [_cell trackRect], 
-						 [_cell isVertical], 
-						 [_cell minValue], 
-						 [_cell maxValue], _cell, 
-						 _rFlags.flipped_view); 
-	  [_cell setFloatValue: floatValue];
-	  if ([_cell isContinuous])
-	    {
-	      [self sendAction: [_cell action] to: [_cell target]];
-	    }
-	  [self lockFocus];
-	  [_cell drawWithFrame: _bounds inView: self];
-	  [self unlockFocus];
-	  [_window flushWindow];
-	}
-      
-      [self trackKnob: theEvent knobRect: rect];
-    }
-}
 @end
