@@ -29,15 +29,72 @@
 #include <Foundation/NSString.h>
 #include <Foundation/NSException.h>
 
-#include "AppKit/NSSliderCell.h"
+#include "AppKit/NSApplication.h"
 #include "AppKit/NSColor.h"
-#include "AppKit/NSGraphics.h"
 #include "AppKit/NSControl.h"
+#include "AppKit/NSEvent.h"
+#include "AppKit/NSGraphics.h"
 #include "AppKit/NSImage.h"
+#include "AppKit/NSSliderCell.h"
 #include "AppKit/NSTextFieldCell.h"
+#include "AppKit/NSWindow.h"
 
 DEFINE_RINT_IF_MISSING
 
+static inline
+float _floatValueForMousePoint (NSPoint point, NSRect knobRect,
+				NSRect slotRect, BOOL isVertical, 
+				float minValue, float maxValue, 
+				NSSliderCell *theCell, BOOL flipped)
+{
+  float floatValue = 0;
+  float position;
+
+  // Adjust the point to lie inside the knob slot. We don't
+  // have to worry whether the view is flipped or not.
+  if (isVertical)
+    {
+      if (point.y < slotRect.origin.y + knobRect.size.height / 2)
+	{
+	  position = slotRect.origin.y + knobRect.size.height / 2;
+	}
+      else if (point.y > slotRect.origin.y + slotRect.size.height
+	- knobRect.size.height / 2)
+	{
+	  position = slotRect.origin.y + slotRect.size.height
+	    - knobRect.size.height / 2;
+	}
+      else
+	position = point.y;
+      // Compute the float value
+      floatValue = (position - (slotRect.origin.y + knobRect.size.height/2))
+	/ (slotRect.size.height - knobRect.size.height);
+      if (flipped)
+	floatValue = 1 - floatValue;
+    }
+  else
+    {
+      if (point.x < slotRect.origin.x + knobRect.size.width / 2)
+	{
+	  position = slotRect.origin.x + knobRect.size.width / 2;
+	}
+      else if (point.x > slotRect.origin.x + slotRect.size.width
+	- knobRect.size.width / 2)
+	{
+	  position = slotRect.origin.x + slotRect.size.width
+	    - knobRect.size.width / 2;
+	}
+      else
+	position = point.x;
+
+      // Compute the float value given the knob size
+      floatValue = (position - (slotRect.origin.x + knobRect.size.width / 2))
+	/ (slotRect.size.width - knobRect.size.width);
+    }
+
+  return floatValue * (maxValue - minValue) + minValue;
+}
+
 /**
   <unit>
   <heading>Class Description</heading>
@@ -136,7 +193,14 @@ DEFINE_RINT_IF_MISSING
 
   floatValue = (floatValue - _minValue) / (_maxValue - _minValue);
 
-  size = [image size];
+  if (image != nil)
+    {
+      size = [image size];
+    }
+  else
+    {
+      size = NSZeroSize;
+    }
 
   if (_isVertical == YES)
     {
@@ -174,6 +238,10 @@ DEFINE_RINT_IF_MISSING
 */
 - (void) drawKnob: (NSRect)knobRect
 {
+  NSColor* knobBackgroundColor = [NSColor controlBackgroundColor];
+  [knobBackgroundColor set];
+  NSRectFill (knobRect);
+  
   [_knobCell drawInteriorWithFrame: knobRect inView: _control_view];
 }
 
@@ -190,16 +258,22 @@ DEFINE_RINT_IF_MISSING
       if (vertical == YES)
 	{
 	  image = [NSImage imageNamed: @"common_SliderVert"];
-	  size = [image size];
-	  [image setScalesWhenResized:YES];
-	  [image setSize: NSMakeSize (cellFrame.size.width, size.height)];
+	  if (image != nil)
+	    {
+	      size = [image size];
+	      [image setScalesWhenResized: YES];
+	      [image setSize: NSMakeSize(cellFrame.size.width, size.height)];
+	    }
 	}
       else
 	{
 	  image = [NSImage imageNamed: @"common_SliderHoriz"];
-	  size = [image size];
-	  [image setScalesWhenResized:YES];
-	  [image setSize: NSMakeSize (size.width, cellFrame.size.height)];
+	  if (image != nil)
+	    {
+	      size = [image size];
+	      [image setScalesWhenResized: YES];
+	      [image setSize: NSMakeSize(size.width, cellFrame.size.height)];
+	    }
 	}
       [_knobCell setImage: image];
     }
@@ -230,8 +304,17 @@ DEFINE_RINT_IF_MISSING
 */
 - (float) knobThickness
 {
-  NSImage	*image = [_knobCell image];
-  NSSize	size = [image size];
+  NSImage *image = [_knobCell image];
+  NSSize size;
+
+  if (image != nil)
+    {
+      size = [image size];
+    }
+  else 
+    {
+	return 0;
+    }
 
   return _isVertical ? size.height : size.width;
 }
@@ -242,8 +325,17 @@ DEFINE_RINT_IF_MISSING
  */
 - (void) setKnobThickness: (float)thickness
 {
-  NSImage	*image = [_knobCell image];
-  NSSize	size = [image size];
+  NSImage *image = [_knobCell image];
+  NSSize size;
+
+  if (image != nil)
+    {
+      size = [image size];
+    }
+  else 
+    {
+      return;
+    }
 
   if (_isVertical == YES)
     size.height = thickness;
@@ -526,18 +618,150 @@ DEFINE_RINT_IF_MISSING
   return _minValue + index * (_maxValue - _minValue) / _numberOfTickMarks;
 }
 
+- (BOOL) trackMouse: (NSEvent*)theEvent
+	     inRect: (NSRect)cellFrame
+	     ofView: (NSView*)controlView
+       untilMouseUp: (BOOL)flag
+{
+  float delay;
+  float interval;
+  id target = [self target];
+  SEL action = [self action];
+  unsigned int eventMask = NSLeftMouseDownMask | NSLeftMouseUpMask
+                           | NSLeftMouseDraggedMask | NSMouseMovedMask;
+  NSEventType eventType = [theEvent type];
+  BOOL isContinuous = [self isContinuous];
+  float oldFloatValue = [self floatValue];
+  NSRect slotRect = [self trackRect];
+  BOOL isVertical = [self isVertical];
+  float minValue = [self minValue];
+  float maxValue = [self maxValue];
+  BOOL isFlipped = [controlView isFlipped];
+  NSPoint location = [theEvent locationInWindow];
+  NSPoint point = [controlView convertPoint: location fromView: nil];
+  NSRect knobRect = [self knobRectFlipped: isFlipped];
+
+  _mouse_down_flags = [theEvent modifierFlags];
+  if (![self isEnabled])
+    {
+      return NO;
+    }
+
+  if (![controlView mouse: point inRect: knobRect])
+    {
+      // Mouse is not on the knob, move the knob to the mouse position
+      float floatValue;
+
+      floatValue = _floatValueForMousePoint(point, knobRect, 
+					    slotRect, isVertical, 
+					    minValue, maxValue,
+					    self, isFlipped); 
+      [self setFloatValue: floatValue];
+      if (isContinuous)
+        {
+	  [(NSControl*)controlView sendAction: action to: target];
+	}
+    }
+      
+  if (isContinuous)
+    {
+      [self getPeriodicDelay: &delay interval: &interval];
+      [NSEvent startPeriodicEventsAfterDelay: delay withPeriod: interval];
+      eventMask |= NSPeriodicMask;
+    }
+
+  while (eventType != NSLeftMouseUp)
+    {
+      theEvent = [NSApp nextEventMatchingMask: eventMask
+			untilDate: nil
+			inMode: NSEventTrackingRunLoopMode
+			dequeue: YES];
+      eventType = [theEvent type];
+
+      if (eventType == NSPeriodic)
+        {
+	  NSWindow *w = [controlView window];
+
+	  location = [w mouseLocationOutsideOfEventStream];
+	}
+      else
+        {
+	  location = [theEvent locationInWindow];
+	}
+      point = [controlView convertPoint: location fromView: nil];
+
+      if (point.x != knobRect.origin.x || point.y != knobRect.origin.y)
+        {
+	  float floatValue;
+
+	  floatValue = _floatValueForMousePoint(point, knobRect,
+						slotRect, isVertical, 
+						minValue, maxValue, 
+						self, isFlipped); 
+	  if (floatValue != oldFloatValue)
+	    {
+	      [self setFloatValue: floatValue];
+	      if (isContinuous)
+	        {
+		  [(NSControl*)controlView sendAction: action to: target];
+		}
+	      oldFloatValue = floatValue;
+	    }
+	  knobRect.origin = point;
+	}
+    }
+
+  // If the cell is not continuous send the action at the end of the drag
+  if (!isContinuous)
+    {
+      [(NSControl*)controlView sendAction: action to: target];
+    }
+  else
+    {
+      [NSEvent stopPeriodicEvents];
+    }
+
+  return YES;
+}
+
 - (id) initWithCoder: (NSCoder*)decoder
 {
   self = [super initWithCoder: decoder];
-  [decoder decodeValuesOfObjCTypes: "fffi",
-    &_minValue, &_maxValue, &_altIncrementValue, &_isVertical];
-  [decoder decodeValueOfObjCType: @encode(id) at: &_titleCell];
-  [decoder decodeValueOfObjCType: @encode(id) at: &_knobCell];
-  if ([decoder versionForClassName: @"NSSliderCell"] >= 2)
+  if([decoder allowsKeyedCoding])
     {
-      [decoder decodeValueOfObjCType: @encode(BOOL) at: &_allowsTickMarkValuesOnly];
-      [decoder decodeValueOfObjCType: @encode(int) at: &_numberOfTickMarks];
-      [decoder decodeValueOfObjCType: @encode(int) at: &_tickMarkPosition];
+      _allowsTickMarkValuesOnly = [decoder decodeBoolForKey: @"NSAllowsTickMarkValuesOnly"];
+      _numberOfTickMarks = [decoder decodeIntForKey: @"NSNumberOfTickMarks"];
+      _tickMarkPosition = [decoder decodeIntForKey: @"NSTickMarkPosition"];
+      _minValue = [decoder decodeFloatForKey: @"NSMinValue"];
+      _maxValue = [decoder decodeFloatForKey: @"NSMaxValue"];
+      _altIncrementValue = [decoder decodeFloatForKey: @"NSAltIncValue"];
+
+      // do these here, since the Cocoa version of the class does not save these values...
+      _knobCell = [NSCell new];
+      _titleCell = [NSTextFieldCell new];
+      [_titleCell setTextColor: [NSColor controlTextColor]];
+      [_titleCell setStringValue: @""];
+      [_titleCell setAlignment: NSCenterTextAlignment];
+
+      // if it's from a nib, make it bordered and bezeled so it's more attractive, this
+      // information is not in the nib.
+      [self setBordered: YES];
+      [self setBezeled: NO];
+
+      _isVertical = -1;
+    }
+  else
+    {
+      [decoder decodeValuesOfObjCTypes: "fffi",
+	       &_minValue, &_maxValue, &_altIncrementValue, &_isVertical];
+      [decoder decodeValueOfObjCType: @encode(id) at: &_titleCell];
+      [decoder decodeValueOfObjCType: @encode(id) at: &_knobCell];
+      if ([decoder versionForClassName: @"NSSliderCell"] >= 2)
+	{
+	  [decoder decodeValueOfObjCType: @encode(BOOL) at: &_allowsTickMarkValuesOnly];
+	  [decoder decodeValueOfObjCType: @encode(int) at: &_numberOfTickMarks];
+	  [decoder decodeValueOfObjCType: @encode(int) at: &_tickMarkPosition];
+	}
     }
   return self;
 }
@@ -545,14 +769,27 @@ DEFINE_RINT_IF_MISSING
 - (void) encodeWithCoder: (NSCoder*)coder
 {
   [super encodeWithCoder: coder];
-  [coder encodeValuesOfObjCTypes: "fffi",
-    &_minValue, &_maxValue, &_altIncrementValue, &_isVertical];
-  [coder encodeValueOfObjCType: @encode(id) at: &_titleCell];
-  [coder encodeValueOfObjCType: @encode(id) at: &_knobCell];
-  // New for version 2
-  [coder encodeValueOfObjCType: @encode(BOOL) at: &_allowsTickMarkValuesOnly];
-  [coder encodeValueOfObjCType: @encode(int) at: &_numberOfTickMarks];
-  [coder encodeValueOfObjCType: @encode(int) at: &_tickMarkPosition];
+  if([coder allowsKeyedCoding])
+    {
+      [coder encodeBool: _allowsTickMarkValuesOnly forKey: @"NSAllowsTickMarkValuesOnly"];
+      [coder encodeInt: _numberOfTickMarks forKey: @"NSNumberOfTickMarks"];
+      [coder encodeInt: _tickMarkPosition forKey: @"NSTickMarkPosition"];
+      [coder encodeFloat: _minValue forKey: @"NSMinValue"];
+      [coder encodeFloat: _maxValue forKey: @"NSMaxValue"];
+      [coder encodeFloat: _altIncrementValue forKey: @"NSAltIncValue"];
+      [coder encodeFloat: _minValue forKey: @"NSValue"]; // encoded for compatibility
+    }
+  else
+    {
+      [coder encodeValuesOfObjCTypes: "fffi",
+	     &_minValue, &_maxValue, &_altIncrementValue, &_isVertical];
+      [coder encodeValueOfObjCType: @encode(id) at: &_titleCell];
+      [coder encodeValueOfObjCType: @encode(id) at: &_knobCell];
+      // New for version 2
+      [coder encodeValueOfObjCType: @encode(BOOL) at: &_allowsTickMarkValuesOnly];
+      [coder encodeValueOfObjCType: @encode(int) at: &_numberOfTickMarks];
+      [coder encodeValueOfObjCType: @encode(int) at: &_tickMarkPosition];
+    }
 }
-
+  
 @end
