@@ -430,10 +430,13 @@
 	      int iFlex = 0;
 	      iP.x = aRect.origin.x;
 	      iP.y = aRect.origin.y;
+	      
+	      [[NSColor controlBackgroundColor] set];
 
 	      if (itemState == NSSelectedTab)
 		{
 		  iP.y += 1;
+	          NSRectFill (NSMakeRect (iP.x, iP.y, 14, 17));
 		  [[NSImage imageNamed: @"common_TabDownSelectedLeft.tiff"]
 		    compositeToPoint: iP operation: NSCompositeSourceOver];
 		  iP.y -= 1;
@@ -442,6 +445,7 @@
 	      else if (itemState == NSBackgroundTab)
 		{
 		  iP.y += 1;
+	          NSRectFill (NSMakeRect (iP.x, iP.y, 14, 17));
 		  [[NSImage imageNamed: @"common_TabDownUnSelectedLeft.tiff"]
 		    compositeToPoint: iP operation: NSCompositeSourceOver];
 		  iP.y -= 1;
@@ -471,11 +475,14 @@
 
 	      iP.x = previousRect.origin.x + previousRect.size.width;
 	      iP.y = aRect.origin.y;
+	      
+	      [[NSColor controlBackgroundColor] set];
 
 	      if (itemState == NSSelectedTab) 
 		{
 		  iP.y += 1;
 		  iFlex = 1;
+	          NSRectFill (NSMakeRect (iP.x, iP.y, 14, 17));
 		  [[NSImage imageNamed:
 		    @"common_TabDownUnSelectedToSelectedJunction.tiff"]
 		    compositeToPoint: iP operation: NSCompositeSourceOver];
@@ -486,6 +493,7 @@
 		  if (previousState == NSSelectedTab)
 		    {
 		      iP.y += 1;
+	              NSRectFill (NSMakeRect (iP.x, iP.y, 14, 17));
 		      [[NSImage imageNamed:
 			@"common_TabDownSelectedToUnSelectedJunction.tiff"]
 			compositeToPoint: iP operation: NSCompositeSourceOver];
@@ -495,6 +503,7 @@
 		  else
 		    {
 		      //		    iP.y += 1;
+	              NSRectFill (NSMakeRect (iP.x, iP.y, 14, 17));
 		      [[NSImage imageNamed:
 			@"common_TabDownUnSelectedJunction.tiff"]
 			compositeToPoint: iP operation: NSCompositeSourceOver];
@@ -528,12 +537,18 @@
 	    {
 	      iP.x += s.width + 13;
 
+	      [[NSColor controlBackgroundColor] set];
+
 	      if ([anItem tabState] == NSSelectedTab)
-		[[NSImage imageNamed: @"common_TabDownSelectedRight.tiff"]
-		  compositeToPoint: iP operation: NSCompositeSourceOver];
+	        {
+	          NSRectFill (NSMakeRect (iP.x, iP.y, 14, 17));
+		  [[NSImage imageNamed: @"common_TabDownSelectedRight.tiff"]
+		    compositeToPoint: iP operation: NSCompositeSourceOver];
+	        }
 	      else if ([anItem tabState] == NSBackgroundTab)
 		{
 		  //		  iP.y += 1;
+	          NSRectFill (NSMakeRect (iP.x, iP.y, 14, 17));
 		  [[NSImage imageNamed: @"common_TabDownUnSelectedRight.tiff"]
 		    compositeToPoint: iP operation: NSCompositeSourceOver];
 		  //		  iP.y -= 1;
@@ -562,16 +577,22 @@
 	    {
 	      iP.x = aRect.origin.x;
 	      iP.y = aRect.size.height;
+
+	      [[NSColor controlBackgroundColor] set];
 	      
 	      if (itemState == NSSelectedTab)
 		{
 		  iP.y -= 1;
+	          NSRectFill (NSMakeRect (iP.x, iP.y, 14, 17));
 		  [[NSImage imageNamed: @"common_TabSelectedLeft.tiff"]
 		    compositeToPoint: iP operation: NSCompositeSourceOver];
 		}
 	      else if (itemState == NSBackgroundTab)
-		[[NSImage imageNamed: @"common_TabUnSelectedLeft.tiff"]
-		  compositeToPoint: iP operation: NSCompositeSourceOver];
+	        {
+	          NSRectFill (NSMakeRect (iP.x, iP.y, 14, 17));
+		  [[NSImage imageNamed: @"common_TabUnSelectedLeft.tiff"]
+		    compositeToPoint: iP operation: NSCompositeSourceOver];
+	        }
 	      else
 		NSLog(@"Not finished yet. Luff ya.\n");
 
@@ -595,10 +616,13 @@
 	    {
 	      iP.x = previousRect.origin.x + previousRect.size.width;
 	      iP.y = aRect.size.height;
+	      
+	      [[NSColor controlBackgroundColor] set];
 
 	      if (itemState == NSSelectedTab)
 		{
 		  iP.y -= 1;
+	          NSRectFill (NSMakeRect (iP.x, iP.y, 14, 17));
 		  [[NSImage imageNamed:
 		    @"common_TabUnSelectToSelectedJunction.tiff"]
 		    compositeToPoint: iP operation: NSCompositeSourceOver];
@@ -608,6 +632,7 @@
 		  if (previousState == NSSelectedTab)
 		    {
 		      iP.y -= 1;
+	              NSRectFill (NSMakeRect (iP.x, iP.y, 14, 17));
 		      [[NSImage imageNamed:
 			@"common_TabSelectedToUnSelectedJunction.tiff"]
 			compositeToPoint: iP operation: NSCompositeSourceOver];
@@ -615,6 +640,7 @@
 		    }
 		  else
 		    {
+	              NSRectFill (NSMakeRect (iP.x, iP.y, 14, 17));
 		      [[NSImage imageNamed:
 			@"common_TabUnSelectedJunction.tiff"]
 			compositeToPoint: iP operation: NSCompositeSourceOver];
@@ -643,13 +669,21 @@
 	  if (i == howMany-1)
 	    {
 	      iP.x += s.width + 13;
+	      
+	      [[NSColor controlBackgroundColor] set];
 	    
 	      if ([anItem tabState] == NSSelectedTab)
-		[[NSImage imageNamed: @"common_TabSelectedRight.tiff"]
-		compositeToPoint: iP operation: NSCompositeSourceOver];
+	        {	      
+	          NSRectFill (NSMakeRect (iP.x, iP.y, 14, 17));
+		  [[NSImage imageNamed: @"common_TabSelectedRight.tiff"]
+		    compositeToPoint: iP operation: NSCompositeSourceOver];
+		}  
 	      else if ([anItem tabState] == NSBackgroundTab)
-		[[NSImage imageNamed: @"common_TabUnSelectedRight.tiff"]
-		compositeToPoint: iP operation: NSCompositeSourceOver];
+		{
+	          NSRectFill (NSMakeRect (iP.x, iP.y, 14, 17));
+		  [[NSImage imageNamed: @"common_TabUnSelectedRight.tiff"]
+		    compositeToPoint: iP operation: NSCompositeSourceOver];
+		}
 	      else
 		NSLog(@"Not finished yet. Luff ya.\n");
 	    }
@@ -666,13 +700,15 @@
 
 // Event handling.
 
+/* 
+ *  Find the tab view item containing the NSPoint point. This point 
+ *  is expected to be alreay in the coordinate system of the tab view.
+ */
 - (NSTabViewItem*) tabViewItemAtPoint: (NSPoint)point
 {
   int		howMany = [_items count];
   int		i;
 
-  point = [self convertPoint: point fromView: nil];
-
   for (i = 0; i < howMany; i++)
     {
       NSTabViewItem *anItem = [_items objectAtIndex: i];
@@ -686,7 +722,8 @@
 
 - (void) mouseDown: (NSEvent *)theEvent
 {
-  NSPoint location = [theEvent locationInWindow];
+  NSPoint location = [self convertPoint: [theEvent locationInWindow] 
+			   fromView: nil];
   NSTabViewItem *anItem = [self tabViewItemAtPoint: location];
   
   if (anItem != nil  &&  ![anItem isEqual: _selected])
@@ -729,14 +766,42 @@
 - (void) encodeWithCoder: (NSCoder*)aCoder
 { 
   [super encodeWithCoder: aCoder];
-           
-  [aCoder encodeObject: _items];
-  [aCoder encodeObject: _font];
-  [aCoder encodeValueOfObjCType: @encode(NSTabViewType) at: &_type];
-  [aCoder encodeValueOfObjCType: @encode(BOOL) at: &_draws_background];
-  [aCoder encodeValueOfObjCType: @encode(BOOL) at: &_truncated_label];
-  [aCoder encodeConditionalObject: _delegate];
-  [aCoder encodeValueOfObjCType: "i" at: &_selected_item];
+  if ([aCoder allowsKeyedCoding])
+    {
+      unsigned int type = 0;
+      switch(_type)
+	{
+	case NSTopTabsBezelBorder:
+	  type = 0;
+	  break;
+	case NSLeftTabsBezelBorder:
+	  type = 1;
+	  break;
+	case NSBottomTabsBezelBorder:
+	  type = 2;
+	  break;
+	case NSRightTabsBezelBorder:
+	  type = 3;
+	  break;
+	}
+
+      [aCoder encodeBool: [self allowsTruncatedLabels] forKey: @"NSAllowTruncatedLabels"];
+      [aCoder encodeBool: [self drawsBackground] forKey: @"NSDrawsBackground"];
+      [aCoder encodeObject: [self font] forKey: @"NSFont"];
+      [aCoder encodeObject: _items forKey: @"NSTabViewItems"];
+      [aCoder encodeObject: [self selectedTabViewItem] forKey: @"NSSelectedTabViewItem"];
+      [aCoder encodeInt: type forKey: @"NSTvFlags"]; // no flags set...
+    }
+  else
+    {
+      [aCoder encodeObject: _items];
+      [aCoder encodeObject: _font];
+      [aCoder encodeValueOfObjCType: @encode(NSTabViewType) at: &_type];
+      [aCoder encodeValueOfObjCType: @encode(BOOL) at: &_draws_background];
+      [aCoder encodeValueOfObjCType: @encode(BOOL) at: &_truncated_label];
+      [aCoder encodeConditionalObject: _delegate];
+      [aCoder encodeValueOfObjCType: "i" at: &_selected_item];
+    }
 }
 
 - (id) initWithCoder: (NSCoder*)aDecoder
@@ -761,14 +826,7 @@
 	}
       if ([aDecoder containsValueForKey: @"NSTabViewItems"])
         {
-	  NSArray *items = [aDecoder decodeObjectForKey: @"NSTabViewItems"];
-	  NSEnumerator *enumerator = [items objectEnumerator];
-	  NSTabViewItem *item;
-	  
-	  while ((item = [enumerator nextObject]) != nil)
-	    {
-	      [self addTabViewItem: item];
-	    }
+	  ASSIGN(_items, [aDecoder decodeObjectForKey: @"NSTabViewItems"]);
 	}
       if ([aDecoder containsValueForKey: @"NSSelectedTabViewItem"])
         {
@@ -777,7 +835,25 @@
 	}
       if ([aDecoder containsValueForKey: @"NSTvFlags"])
         {
-	    //int flags = [aDecoder decodeObjectForKey: @"NSTvFlags"]];
+	  unsigned int type = [aDecoder decodeIntForKey: @"NSTvFlags"];
+	  switch(type)
+	    {
+	    case 0: 
+	      _type = NSTopTabsBezelBorder;
+	      break;
+	    case 1: 
+	      _type = NSLeftTabsBezelBorder;
+	      break;
+	    case 2: 
+	      _type = NSBottomTabsBezelBorder;
+	      break;
+	    case 3:
+	      _type = NSRightTabsBezelBorder;
+	      break;
+	    default:
+	      _type = NSTopTabsBezelBorder;
+	      break;
+	    }
 	}
     }
   else
