@@ -25,7 +25,7 @@
    You should have received a copy of the GNU Library General Public
    License along with this library; see the file COPYING.LIB.
    If not, write to the Free Software Foundation,
-   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #include <Foundation/NSArray.h>
@@ -74,7 +74,7 @@
  */
 
 // TODO: Remove this method, if not really needed
-- (NSTableHeaderView*)initWithFrame:(NSRect)frameRect
+- (id)initWithFrame:(NSRect)frameRect
 {
   self = [super initWithFrame: frameRect];
   _tableView = nil;
