@@ -305,31 +305,32 @@ static BOOL classInheritsFromNSMutableAttributedString (Class c)
 	       documentAttributes: (NSDictionary **)dict
 			    class: (Class)class
 {
-  RTFConsumer *consumer = [RTFConsumer new];
   NSAttributedString *text = nil;
 
   if ([wrapper isRegularFile])
     {
+      RTFConsumer *consumer = [RTFConsumer new];
       text = [consumer parseRTF: [wrapper regularFileContents]
 		       documentAttributes: dict
 		       class: class];
+      RELEASE(consumer);
     }
   else if ([wrapper isDirectory])
     {
       NSDictionary *files = [wrapper fileWrappers];
       NSFileWrapper *contents;
+      RTFDConsumer* consumer = [RTFDConsumer new];
 
-      //FIXME: We should store the files in the consumer
-      // We try to read the main file in the directory
       if ((contents = [files objectForKey: @"TXT.rtf"]) != nil)
 	{
+	  [consumer setFiles: files];
 	  text = [consumer parseRTF: [contents regularFileContents]
 			   documentAttributes: dict
 			   class: class];
 	}
+      RELEASE(consumer);
     }
   
-  RELEASE(consumer);
 
   return text;
 }
@@ -374,8 +375,35 @@ static BOOL classInheritsFromNSMutableAttributedString (Class c)
 
 @end
 
+
+@interface RTFDConsumer (Private)
+
+- (void) appendImage: (NSString*) string;
+
+@end
+
 @implementation RTFDConsumer
 
+- (id) init
+{
+  self = [super init];
+
+  files = nil;
+
+  return self;
+}
+
+- (void) dealloc
+{
+  RELEASE(files);
+  [super dealloc];
+}
+
+- (void) setFiles: (NSDictionary*) theFiles
+{
+  ASSIGN (files, theFiles);
+}
+
 + (NSAttributedString*) parseData: (NSData *)rtfData 
 	       documentAttributes: (NSDictionary **)dict
 			    class: (Class)class
@@ -390,6 +418,49 @@ static BOOL classInheritsFromNSMutableAttributedString (Class c)
   return str;
 }
 
+- (void) appendImage: (NSString*)string
+{
+  int  oldPosition = [result length];
+  NSRange insertionRange = NSMakeRange(oldPosition,0);
+
+  if (!ignore)
+    {
+      NSString* fileName = [string stringByTrimmingCharactersInSet:
+                         [NSCharacterSet whitespaceAndNewlineCharacterSet]];
+      NSFileWrapper* wrapper = [files objectForKey: fileName];
+      
+      if (wrapper != nil)
+        {
+          NSImage* image = [[NSImage alloc] initWithData: [wrapper regularFileContents]];
+          NSTextAttachmentCell* attachedCell = [[NSTextAttachmentCell alloc] initImageCell: image];
+          NSTextAttachment* attachment = [[NSTextAttachment alloc] initWithFileWrapper: wrapper];
+          RTFAttribute* attr = [self attr];
+          NSMutableDictionary* attributes = nil;
+          NSMutableAttributedString* str = nil;
+
+          [attachment setAttachmentCell: attachedCell];
+        
+          attributes = [[NSMutableDictionary alloc]
+			 initWithObjectsAndKeys:
+			   [attr currentFont], NSFontAttributeName,
+			   attr->paragraph, NSParagraphStyleAttributeName,
+			   nil];
+          
+          str = (NSMutableAttributedString*) [NSMutableAttributedString 
+                         attributedStringWithAttachment: attachment];
+
+          [str addAttributes: attributes range: NSMakeRange (0, [str length])];
+          
+          [result replaceCharactersInRange: insertionRange withAttributedString: str];
+          
+          RELEASE(attributes);
+          RELEASE(attachment);
+          RELEASE(attachedCell);
+          RELEASE(image);
+        }
+    }
+}
+
 @end
 
 @implementation RTFConsumer (Private)
@@ -552,6 +623,8 @@ static BOOL classInheritsFromNSMutableAttributedString (Class c)
 #define TEXTPOSITION [RESULT length]
 #define DOCUMENTATTRIBUTES ((RTFConsumer*)ctxt)->documentAttributes
 
+#define FILES ((RTFDConsumer*)ctxt)->files
+
 #define CTXT [((RTFConsumer *)ctxt) attr]
 #define CHANGED CTXT->changed
 #define PARAGRAPH CTXT->paragraph
@@ -1029,3 +1102,9 @@ void GSRTFparagraph (void *ctxt)
   GSRTFmangleText(ctxt, "\n");
   CTXT->tabChanged = NO;
 }
+
+void GSRTFNeXTGraphic (void *ctxt, const char *fileName, int width, int height)
+{
+  [(RTFDConsumer *)ctxt appendImage: [NSString stringWithCString: fileName]];
+}
+
